# Copyright (c) 2011 Nicira, Inc.
# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVELogger import *
import ovs.json

def str_recursive(x):
    t = type(x)
    if t == unicode:
        return str(x)
    elif t == list:
        return [str_recursive(_) for _ in x]
    elif t == dict:
        out = {}
        for k,v in x.iteritems():
            out[str_recursive(k)] = str_recursive(v)
        return out
    else:
        return x

class OVEConfig(QtCore.QObject):
    instance = None
    def __init__(self):
        QtCore.QObject.__init__(self)
        self.hosts = []
        self.logTraffic = True
        self.truncateUuids = True
        self.ssgList = []
        
    @classmethod
    def Inst(cls):
        if cls.instance is None:
            cls.instance = OVEConfig()
            cls.instance.loadConfig()
        return cls.instance

    def hostFromUuid(self, uuid):
        for host in self.hosts:
            if host['uuid'] == uuid:
                return host
        OVELog("+++ Couldn't find host '"+str(uuid)+"' in "+str([x['uuid'] for x in self.hosts]))
        return None

    def saveConfig(self):
        settings = QtCore.QSettings()
        settings.setValue('config/hosts', QVariant(ovs.json.to_string((self.hosts))))
        settings.setValue('config/logTraffic', QVariant(self.logTraffic))
        settings.setValue('config/truncateUuids', QVariant(self.truncateUuids))
        settings.setValue('config/ssgList', QVariant(ovs.json.to_string(self.ssgList)))
        settings.sync()
        self.emitUpdated()

    def loadConfig(self):
        settings = QtCore.QSettings()
        jsonText = unicode(settings.value('config/hosts', QVariant('[]')).toString())
        self.hosts = str_recursive(ovs.json.from_string(str(jsonText)))
        self.logTraffic = settings.value('config/logTraffic', QVariant(False)).toBool()
        self.truncateUuids = settings.value('config/truncateUuids', QVariant(False)).toBool()
        jsonText = unicode(settings.value('config/ssgList', QVariant('[]')).toString())
        self.ssgList = ovs.json.from_string(str(jsonText))
        if len(self.ssgList) == 0:
            self.ssgList = [
                r'in_port0000',
                r'in_port0001',
                r'in_port0002',
                r'in_port0003',
                r'vlan65535',
                r'type0800',
                r'type0806',
                r'proto0',
                r'proto6',
                r'proto17',
                r'ff:ff:ff:ff:ff:ff',
                r'!ff:ff:ff:ff:ff:ff',
                r'0\.0\.0\.0',
                r'!0\.0\.0\.0',
                r'255\.255\.255\.255',
                r'!255\.255\.255\.255',
                r'never',
                r'drop',
                r'!never',
                r'!drop',
                r'(never|drop)',
                r'!(never|drop)'
            ]
        
    def emitUpdated(self):
        self.emit(QtCore.SIGNAL("configUpdated()"))



# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *

class OVELogger(QtCore.QObject):
    instance = None
    def __init__(self):
        QtCore.QObject.__init__(self)
        self.contents = []
        self.loggers = []
        
    @classmethod
    def Inst(cls):
        if cls.instance is None:
            cls.instance = OVELogger()
        return cls.instance
    
    def reset(self):
        self.contents = []
        self.update()
    
    def logString(self, message):
        self.contents += [str(message)]
        if len(self.contents) > 500:
            self.contents = ['+++ Log truncated', ''] + self.contents[50:]
        self.update()

    def update(self):
        self.emit(QtCore.SIGNAL("logUpdated()"))
        
def OVELog(message):
    OVELogger.Inst().logString(message)
    



# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVEConfig import *
from OVELogger import *
from Ui_ConfigWindow import *

from OVEHostWindow import *

class OVEConfigWindow(QtGui.QDialog):
    def __init__(self, app):
        QtGui.QDialog.__init__(self)
        self.app = app
        self.ui = Ui_ConfigWindow()
        self.ui.setupUi(self)

        self.connect(self.ui.hostAddButton, QtCore.SIGNAL("clicked()"), self.xon_hostAddButton_clicked)
        self.connect(self.ui.hostEditButton, QtCore.SIGNAL("clicked()"), self.xon_hostEditButton_clicked)
        self.connect(self.ui.hostDeleteButton, QtCore.SIGNAL("clicked()"), self.xon_hostDeleteButton_clicked)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("clicked(QAbstractButton *)"), self.xon_actionButton_Box_clicked)
        self.connect(self.ui.hostList, QtCore.SIGNAL("currentItemChanged(QListWidgetItem *,  QListWidgetItem *)"), self.xon_hostList_currentItemChanged)
        self.connect(self.ui.logTrafficCheckBox, QtCore.SIGNAL("stateChanged(int)"), self.xon_logTrafficCheckBox_stateChanged)
        self.connect(self.ui.truncateUuidsCheckBox, QtCore.SIGNAL("stateChanged(int)"), self.xon_truncateUuidsCheckBox_stateChanged)
        self.readConfig()
        self.updateWidgets()
    
    def handleHostWindowRecord(self, record, isEdit):
        if record['accepted'] and record['address'].strip() != '':
            currentRow = self.ui.hostList.currentRow()
            if isEdit:
                self.configHosts[currentRow] = record
            else:
                self.configHosts.append(record)

        self.updateWidgets()

    def xon_hostAddButton_clicked(self):
        hostWindow = OVEHostWindow(self)
        hostWindow.exec_()
        self.handleHostWindowRecord(hostWindow.record(), False)

    def xon_hostEditButton_clicked(self):
        if self.ui.hostList.currentItem() is None:
            pass # OVELog('No item to edit')
        else:
            currentRow = self.ui.hostList.currentRow()
            hostWindow = OVEHostWindow(self, self.configHosts[currentRow])
            hostWindow.exec_()
            self.handleHostWindowRecord(hostWindow.record(), True)

    def xon_hostDeleteButton_clicked(self):
        if self.ui.hostList.currentItem() is not None:
            currentRow = self.ui.hostList.currentRow()
            del self.configHosts[currentRow]
            self.updateWidgets()

    def xon_actionButton_Box_clicked(self, button):
        role = self.ui.buttonBox.buttonRole(button)
        if role == QtGui.QDialogButtonBox.AcceptRole:
            self.writeConfig()
            self.close()
        elif role == QtGui.QDialogButtonBox.ApplyRole:
            self.writeConfig()
        elif role == QtGui.QDialogButtonBox.RejectRole:
            if self.configChanged():
                self.close()
            else:
                ret = QtGui.QMessageBox.warning(
                    self, "OVSDB Monitor",
                    "Changes not applied. Discard?",
                    QtGui.QMessageBox.Discard | QtGui.QMessageBox.Cancel | QtGui.QMessageBox.Apply,
                    QtGui.QMessageBox.Discard)
                
                if ret == QtGui.QMessageBox.Apply:
                    self.writeConfig()
                if ret != QtGui.QMessageBox.Cancel:
                    self.close()

    def xon_hostList_currentItemChanged(self, current, previous):
        editable = (current is not None)
        self.ui.hostEditButton.setEnabled(editable)
        self.ui.hostDeleteButton.setEnabled(editable)

    def xon_logTrafficCheckBox_stateChanged(self, value):
        self.configLogTraffic = (value == Qt.Checked)
    
    def xon_truncateUuidsCheckBox_stateChanged(self, value):
        self.configTruncateUuids = (value == Qt.Checked)
    
    def updateWidgets(self):
        self.ui.hostList.clear()
        for host in self.configHosts:
            self.ui.hostList.addItem(host['address'])
        self.ui.logTrafficCheckBox.setChecked(self.configLogTraffic)
        self.ui.truncateUuidsCheckBox.setChecked(self.configTruncateUuids)

    def configChanged(self):
        return (
            (self.configHosts == OVEConfig.Inst().hosts) and
            (self.configLogTraffic == (OVEConfig.Inst().logTraffic))and
            (self.configTruncateUuids == (OVEConfig.Inst().truncateUuids))
        )
        
    def readConfig(self):
        self.configHosts = deepcopy(OVEConfig.Inst().hosts)
        self.configLogTraffic = OVEConfig.Inst().logTraffic
        self.configTruncateUuids = OVEConfig.Inst().truncateUuids
        
    def writeConfig(self):
        OVEConfig.Inst().hosts = deepcopy(self.configHosts)
        OVEConfig.Inst().logTraffic = self.configLogTraffic
        OVEConfig.Inst().truncateUuids = self.configTruncateUuids
        OVEConfig.Inst().saveConfig()

    



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'FlowWindow.ui'
#
# Created: Fri May  7 17:20:33 2010
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

try:
    from OVEStandard import globalForcePySide
    if globalForcePySide: raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_FlowWindow(object):
    def setupUi(self, FlowWindow):
        FlowWindow.setObjectName("FlowWindow")
        FlowWindow.resize(800,600)
        self.centralwidget = QtGui.QWidget(FlowWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.unset = QtGui.QWidget()
        self.unset.setObjectName("unset")
        self.gridLayout_10 = QtGui.QGridLayout(self.unset)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.tabWidget.addTab(self.unset,"")
        self.gridLayout.addWidget(self.tabWidget,0,0,1,1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.ssgCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.ssgCheckBox.setObjectName("ssgCheckBox")
        self.horizontalLayout_2.addWidget(self.ssgCheckBox)
        self.ssgComboBox = QtGui.QComboBox(self.centralwidget)
        self.ssgComboBox.setEditable(True)
        self.ssgComboBox.setMaxVisibleItems(20)
        self.ssgComboBox.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.ssgComboBox.setMinimumContentsLength(32)
        self.ssgComboBox.setObjectName("ssgComboBox")
        self.horizontalLayout_2.addWidget(self.ssgComboBox)
        self.ssgSaveButton = QtGui.QPushButton(self.centralwidget)
        self.ssgSaveButton.setObjectName("ssgSaveButton")
        self.horizontalLayout_2.addWidget(self.ssgSaveButton)
        self.ssgDeleteButton = QtGui.QPushButton(self.centralwidget)
        self.ssgDeleteButton.setObjectName("ssgDeleteButton")
        self.horizontalLayout_2.addWidget(self.ssgDeleteButton)
        spacerItem = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout_2,1,0,1,1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.hostLabel = QtGui.QLabel(self.centralwidget)
        self.hostLabel.setObjectName("hostLabel")
        self.horizontalLayout.addWidget(self.hostLabel)
        self.hostComboBox = QtGui.QComboBox(self.centralwidget)
        self.hostComboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.hostComboBox.setObjectName("hostComboBox")
        self.horizontalLayout.addWidget(self.hostComboBox)
        self.intervalCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.intervalCheckBox.setObjectName("intervalCheckBox")
        self.horizontalLayout.addWidget(self.intervalCheckBox)
        self.intervalSpinBox = QtGui.QSpinBox(self.centralwidget)
        self.intervalSpinBox.setMinimum(1)
        self.intervalSpinBox.setMaximum(1000000)
        self.intervalSpinBox.setObjectName("intervalSpinBox")
        self.horizontalLayout.addWidget(self.intervalSpinBox)
        spacerItem1 = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.fetchPathsButton = QtGui.QPushButton(self.centralwidget)
        self.fetchPathsButton.setObjectName("fetchPathsButton")
        self.horizontalLayout.addWidget(self.fetchPathsButton)
        self.fetchButton = QtGui.QPushButton(self.centralwidget)
        self.fetchButton.setObjectName("fetchButton")
        self.horizontalLayout.addWidget(self.fetchButton)
        self.gridLayout.addLayout(self.horizontalLayout,3,0,1,1)
        self.line = QtGui.QFrame(self.centralwidget)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line,2,0,1,1)
        FlowWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(FlowWindow)
        self.menubar.setGeometry(QtCore.QRect(0,0,800,28))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        FlowWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(FlowWindow)
        self.statusbar.setObjectName("statusbar")
        FlowWindow.setStatusBar(self.statusbar)
        self.actionShow_Log = QtGui.QAction(FlowWindow)
        self.actionShow_Log.setObjectName("actionShow_Log")
        self.actionNew_DB_Window = QtGui.QAction(FlowWindow)
        self.actionNew_DB_Window.setObjectName("actionNew_DB_Window")
        self.actionPreferences = QtGui.QAction(FlowWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionQuit = QtGui.QAction(FlowWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionNew_Flow_Window = QtGui.QAction(FlowWindow)
        self.actionNew_Flow_Window.setObjectName("actionNew_Flow_Window")
        self.menuFile.addAction(self.actionNew_DB_Window)
        self.menuFile.addAction(self.actionNew_Flow_Window)
        self.menuFile.addAction(self.actionShow_Log)
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.hostLabel.setBuddy(self.hostComboBox)

        self.retranslateUi(FlowWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(FlowWindow)

    def retranslateUi(self, FlowWindow):
        FlowWindow.setWindowTitle(QtGui.QApplication.translate("FlowWindow", "OVSDB Monitor", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.unset), QtGui.QApplication.translate("FlowWindow", "Awaiting update...", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgCheckBox.setText(QtGui.QApplication.translate("FlowWindow", "Server-side grep", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgSaveButton.setText(QtGui.QApplication.translate("FlowWindow", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.ssgDeleteButton.setText(QtGui.QApplication.translate("FlowWindow", "Delete", None, QtGui.QApplication.UnicodeUTF8))
        self.hostLabel.setText(QtGui.QApplication.translate("FlowWindow", "Host", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalCheckBox.setText(QtGui.QApplication.translate("FlowWindow", "Auto-refetch every", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalSpinBox.setSuffix(QtGui.QApplication.translate("FlowWindow", "s", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchPathsButton.setToolTip(QtGui.QApplication.translate("FlowWindow", "Refetches the datapath names and rebuilds the window tabs to reflect them.  Use when the network has been reconfigured, e.g. a bond has been created", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchPathsButton.setText(QtGui.QApplication.translate("FlowWindow", "Refetch Datapath List", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchButton.setText(QtGui.QApplication.translate("FlowWindow", "Refetch", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("FlowWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow_Log.setText(QtGui.QApplication.translate("FlowWindow", "Show Log", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_DB_Window.setText(QtGui.QApplication.translate("FlowWindow", "New DB Window", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPreferences.setText(QtGui.QApplication.translate("FlowWindow", "Preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("FlowWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Flow_Window.setText(QtGui.QApplication.translate("FlowWindow", "New Flow Window", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) 2011 Nicira, Inc.
# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVEConfig import *
from OVEFetch import *
from OVELogger import *
from OVEUtil import *

from OVECommonWindow import *

from Ui_FlowWindow import *

import re

class OVEFlowWindow(QtGui.QMainWindow, OVECommonWindow):
    LOAD_KEY = 'FlowWindow/window'
    COMMAND_OVS_DPCTL='/usr/bin/ovs-dpctl'
    BASE_REF=200000
    
    def __init__(self, app, loadIndex = None):
        QtGui.QMainWindow.__init__(self)
        self.ui = Ui_FlowWindow()
        self.dpNames = []
        self.dpTables = []
        self.currentOpIndex = None
        self.resizeCount = []
        self.ssgChecked = False
        self.ssgText = ''
        self.lastTime = None
        self.lastByteCount = 0
        OVECommonWindow.__init__(self, app, loadIndex)

        self.updateSsgList()
        self.updateDatapaths()
        self.updateSsgState()

        self.connect(self.ui.fetchPathsButton, QtCore.SIGNAL("clicked()"), self.xon_fetchPathsButton_clicked)
        self.connect(self.ui.ssgSaveButton, QtCore.SIGNAL("clicked()"), self.xon_ssgSaveButton_clicked)
        self.connect(self.ui.ssgDeleteButton, QtCore.SIGNAL("clicked()"), self.xon_ssgDeleteButton_clicked)
        self.connect(self.ui.ssgComboBox, QtCore.SIGNAL("activated(int)"), self.xon_ssgComboBox_activated)
        self.connect(self.ui.ssgComboBox, QtCore.SIGNAL("editTextChanged(QString)"), self.xon_ssgComboBox_editTextChanged)
        self.connect(self.ui.ssgCheckBox, QtCore.SIGNAL("stateChanged(int)"), self.xon_ssgCheckBox_stateChanged)
        
    
    def xon_fetchPathsButton_clicked(self):
        self.updateDatapaths()
    
    def xon_hostComboBox_currentIndexChanged(self, index):
        OVECommonWindow.xon_hostComboBox_currentIndexChanged(self, index)
        if (index >= 0):
            self.updateDatapaths()
    
    def xon_ssgSaveButton_clicked(self):
        if self.ssgText not in OVEConfig.Inst().ssgList:
            OVEConfig.Inst().ssgList.append(self.ssgText)
            OVEConfig.Inst().saveConfig()
        self.updateSsgList()
        
    def updateSsgList(self):
        currentSsgText = self.ssgText
        self.ui.ssgComboBox.clear()
        isFound = False
        for i, ssgText in enumerate(OVEConfig.Inst().ssgList):
            self.ui.ssgComboBox.addItem(ssgText)
            if ssgText == currentSsgText:
                # This is the currently selected item
                self.ui.ssgComboBox.setCurrentIndex(i)
                isFound = True
        
        if not isFound:
            self.ui.ssgComboBox.setCurrentIndex(-1)
            self.ui.ssgComboBox.lineEdit().setText(currentSsgText)

    def xon_ssgDeleteButton_clicked(self):
        if self.ssgText in OVEConfig.Inst().ssgList:
            OVEConfig.Inst().ssgList.remove(self.ssgText)
            self.ssgText = ''
            OVEConfig.Inst().saveConfig()
        self.updateSsgList()

    def xon_ssgComboBox_activated(self, index):
        if (index >= 0):
            itemData = self.ui.ssgComboBox.itemText(index)
            self.ssgText = str(itemData)
            self.updateTable()
    
    def xon_ssgComboBox_editTextChanged(self, text):
        self.ssgText = str(text)
        self.statusBar().showMessage('Remote command is: '+self.updateCommand())
        present = (self.ssgText in OVEConfig.Inst().ssgList)
        self.ui.ssgDeleteButton.setEnabled(present)
        self.ui.ssgSaveButton.setEnabled(not present)
    
    def xon_ssgCheckBox_stateChanged(self, state):
        self.ssgChecked = (state == Qt.Checked)
        self.updateTable()
    
    def xon_configUpdated(self):
        OVECommonWindow.xon_configUpdated(self)
        self.updateSsgList()
        self.updateDatapaths()
    
    def timerEvent(self, event):
        OVECommonWindow.timerEvent(self, event)

    def customEvent(self, event):
        OVECommonWindow.customEvent(self, event)
    
    def updateDatapaths(self):
        if self.hostUuid == '':
            self.statusBar().showMessage('No host selected')
        else:
            self.currentRef += 1
            self.currentOp = 'dump-dps'
            command = self.COMMAND_OVS_DPCTL+' dump-dps'
            OVEFetch.Inst(self.hostUuid).execCommandFramed(self, self.currentRef, command)
    
    def rebuildTables(self):
        self.ui.tabWidget.clear() # Let the garbage collector delete the pages
        self.dpTables = []
        self.dpFlows = []
        self.resizeCount = []
        headings = OVEUtil.flowDecodeHeadings()
        
        for dpName in self.dpNames:
            pageWidget = QtGui.QWidget()
            pageWidget.setObjectName(dpName+'_page')
            gridLayout = QtGui.QGridLayout(pageWidget)
            gridLayout.setObjectName(dpName+"_gridLayout")
            table = QtGui.QTableWidget(pageWidget)
            table.setObjectName(dpName+"_table")
            table.setColumnCount(len(headings))
            table.setRowCount(0)
            gridLayout.addWidget(table, 0, 0, 1, 1)
            self.dpTables.append(table)
            self.ui.tabWidget.addTab(pageWidget, dpName)
            self.dpFlows.append([])
            self.resizeCount.append(0)
            for i, heading in enumerate(headings):
                table.setHorizontalHeaderItem(i, QtGui.QTableWidgetItem(heading))

            table.setSortingEnabled(True)
            
            table.sortItems(OVEUtil.getFlowColumn('source mac'))
            table.setSelectionMode(QtGui.QAbstractItemView.NoSelection)
    
    def updateSsgState(self):
        self.ui.ssgCheckBox.setChecked(self.ssgChecked)
    
    def updateCommand(self, overrideText = None):
        command = self.COMMAND_OVS_DPCTL+' dump-flows '
        if self.currentOpIndex is not None:
            command += self.dpNames[self.currentOpIndex]
        exp = None
        if overrideText is not None:
                exp = overrideText
        elif self.ssgChecked:
                exp = self.ssgText
                
        if exp is not None:
            opts='-E '
            if exp.startswith('!'):
                exp =exp[1:]
                opts += '-v '
            command += " | grep "+opts+"'"+exp+"' ; test ${PIPESTATUS[0]} -eq 0 "

        return command
    
    def updateTable(self):
        if self.hostUuid == '':
            self.statusBar().showMessage('No host selected')
            self.setWindowTitle('OVS Flows')
        elif len(self.dpNames) > 0:
            config = OVEConfig.Inst().hostFromUuid(self.hostUuid)
            self.setWindowTitle('OVS Flows - '+config.get('address', ''))
            try:
                self.setFetchSkip()
                self.statusBar().showMessage('Fetching data...') 
                self.currentRef += 1
                self.currentOp = 'dump-flows'
                self.currentOpIndex = self.ui.tabWidget.currentIndex()
                OVEFetch.Inst(self.hostUuid).execCommandFramed(self, self.currentRef, self.updateCommand())
            except Exception, e:
                message = 'Update failed: '+str(e)
                OVELog(message)
                self.statusBar().showMessage(message)
    
    def writeCurrentTable(self):
        index = self.ui.tabWidget.currentIndex()
        actionsColumn = OVEUtil.getFlowColumn('actions')
        usedColumn = OVEUtil.getFlowColumn('used')
        srcMacColumn = OVEUtil.getFlowColumn('source mac')
        destMacColumn = OVEUtil.getFlowColumn('destination mac')
        srcIPColumn = OVEUtil.getFlowColumn('source ip')
        destIPColumn = OVEUtil.getFlowColumn('destination ip')
        inportColumn = OVEUtil.getFlowColumn('inport')
        vlanColumn = OVEUtil.getFlowColumn('vlan')
        bytesColumn = OVEUtil.getFlowColumn('bytes')
        
        byteCount = 0
        try:
            table = self.dpTables[index]
            table.setUpdatesEnabled(False)
            table.setSortingEnabled(False)
            try:
                flows = self.dpFlows[index]
                table.setRowCount(len(flows))
                if len(flows) > 0:
                    table.setColumnCount(len(flows[0]))
                for rowNum, flow in enumerate(flows):
                    
                    inport = flow[inportColumn]
                    if flow[actionsColumn] == 'drop':
                        baseLum=172
                    else:
                        baseLum=239
                    background = QtGui.QColor(baseLum+16*(inport % 2), baseLum+8*(inport % 3), baseLum+4*(inport % 5))
                    if flow[usedColumn] == 'never':
                        colour = QtGui.QColor(112,112,112)
                    else:
                        colour = Qt.black
                        
                    for colNum, data in enumerate(flow):
                        item = None
                        try:
                            item = table.takeItem(rowNum, colNum)
                        except:
                            pass
                        if item is None:
                            item = QtGui.QTableWidgetItem('')

                        if colNum == vlanColumn:
                            item.setBackground(QtGui.QColor(255-(10*data % 192), 255-((17*data) % 192), 255-((37*data) % 192)))
                        elif colNum == srcMacColumn or colNum == destMacColumn:
                            cols = [int(x, 16) for x in data.split(':')]
                            item.setBackground(QtGui.QColor(255-cols[2]*cols[3] % 192, 255-cols[3]*cols[4] % 192, 255-cols[4]*cols[5] % 192))
                        elif re.match(r'[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+', str(data)):
                            cols = [int(x) for x in data.split('.')]
                            item.setBackground(QtGui.QColor(255-cols[1]*cols[2] % 192, 255-cols[2]*cols[3] % 192, 255-cols[3]*cols[0] % 192))
                        else:
                            item.setBackground(background)
                            item.setForeground(colour)
                            
                        if colNum == bytesColumn:
                            byteCount += int(data)
                            
                        # PySide 0.2.3 fails to convert long ints to QVariants and logs 'long int too large to convert to int' errors
                        try:
                            item.setData(Qt.DisplayRole, QVariant(data))
                            item.setToolTip(str(data))
                        except Exception, e:
                            item.setText('Error: See tooltip')
                            item.setToolTip(str(e))
                        table.setItem(rowNum, colNum, item)
                
                if self.resizeCount[index] < 2:
                    self.resizeCount[index] += 1
                    for i in range(0, table.columnCount()):
                        table.resizeColumnToContents(i)

            finally:
                table.setUpdatesEnabled(True)
                table.setSortingEnabled(True)

            message = 'Updated at '+str(QtCore.QTime.currentTime().toString())
            
            if self.lastTime is not None:
                timeDiff = time.time() - self.lastTime
                byteDiff = byteCount - self.lastByteCount
                bitRate = long(8 * byteDiff / timeDiff)
                if abs(bitRate) < 10*2**20:
                    message += ' ('+str(bitRate/2**10)+' kbit/s)'
                elif abs(bitRate) < 10*2**30:
                    message += ' ('+str(bitRate/2**20)+' Mbit/s)'
                else:
                    message += ' ('+str(bitRate/2**30)+' Gbit/s)'
                
            self.lastByteCount = byteCount
            self.lastTime = time.time()
            if table.rowCount() == 0:
                message += ' - Table is empty'
            self.statusBar().showMessage(message)

        except Exception, e:
            message = 'Table update failed: '+str(e)
            OVELog(message)
            self.statusBar().showMessage(message)

    def handleFetchEvent(self, ref, values):
        if self.currentOp == 'dump-dps':
            self.dpNames =values.strip().split('\n')
            self.rebuildTables()
            self.updateTable()
        elif self.currentOp == 'dump-flows':
            self.dpFlows[self.currentOpIndex] = OVEUtil.decodeFlows(values)
            self.writeCurrentTable()

    def handleFetchFailEvent(self, ref, message):
        self.statusBar().showMessage(message)
        OVELog('Fetch ('+self.currentOp+') failed')

    def customEvent(self, event):
        OVECommonWindow.customEvent(self, event)
        
    def saveSettings(self, index):
        settings, key = OVECommonWindow.saveSettings(self, index)
        settings.setValue(key+"/ssgText", QVariant(self.ssgText))
        settings.setValue(key+"/ssgChecked", QVariant(self.ssgChecked))
        
    def loadSettings(self, index):
        settings, key = OVECommonWindow.loadSettings(self, index)
        self.ssgText = str(settings.value(key+"/ssgText", QVariant('10\.80\.226\..*')).toString())
        self.ssgChecked = settings.value(key+"/ssgChecked", QVariant(False)).toBool()
        self.ssgRe = re.compile(self.ssgText)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ConfigWindow.ui'
#
# Created: Fri May  7 17:20:33 2010
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

try:
    from OVEStandard import globalForcePySide
    if globalForcePySide: raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_ConfigWindow(object):
    def setupUi(self, ConfigWindow):
        ConfigWindow.setObjectName("ConfigWindow")
        ConfigWindow.resize(386,303)
        ConfigWindow.setFocusPolicy(QtCore.Qt.TabFocus)
        self.gridLayout = QtGui.QGridLayout(ConfigWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtGui.QTabWidget(ConfigWindow)
        self.tabWidget.setObjectName("tabWidget")
        self.hosts = QtGui.QWidget()
        self.hosts.setObjectName("hosts")
        self.layoutWidget = QtGui.QWidget(self.hosts)
        self.layoutWidget.setGeometry(QtCore.QRect(10,10,341,194))
        self.layoutWidget.setObjectName("layoutWidget")
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.hostList = QtGui.QListWidget(self.layoutWidget)
        self.hostList.setObjectName("hostList")
        self.horizontalLayout_2.addWidget(self.hostList)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.hostAddButton = QtGui.QPushButton(self.layoutWidget)
        self.hostAddButton.setObjectName("hostAddButton")
        self.verticalLayout_2.addWidget(self.hostAddButton)
        self.hostEditButton = QtGui.QPushButton(self.layoutWidget)
        self.hostEditButton.setObjectName("hostEditButton")
        self.verticalLayout_2.addWidget(self.hostEditButton)
        self.hostDeleteButton = QtGui.QPushButton(self.layoutWidget)
        self.hostDeleteButton.setObjectName("hostDeleteButton")
        self.verticalLayout_2.addWidget(self.hostDeleteButton)
        spacerItem = QtGui.QSpacerItem(20,40,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.tabWidget.addTab(self.hosts,"")
        self.logging = QtGui.QWidget()
        self.logging.setObjectName("logging")
        self.gridLayout_2 = QtGui.QGridLayout(self.logging)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.logTrafficCheckBox = QtGui.QCheckBox(self.logging)
        self.logTrafficCheckBox.setObjectName("logTrafficCheckBox")
        self.gridLayout_2.addWidget(self.logTrafficCheckBox,0,0,1,1)
        spacerItem1 = QtGui.QSpacerItem(20,164,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1,1,0,1,1)
        self.tabWidget.addTab(self.logging,"")
        self.view = QtGui.QWidget()
        self.view.setObjectName("view")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.view)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.truncateUuidsCheckBox = QtGui.QCheckBox(self.view)
        self.truncateUuidsCheckBox.setObjectName("truncateUuidsCheckBox")
        self.verticalLayout_3.addWidget(self.truncateUuidsCheckBox)
        spacerItem2 = QtGui.QSpacerItem(20,164,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.tabWidget.addTab(self.view,"")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem3 = QtGui.QSpacerItem(40,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.buttonBox = QtGui.QDialogButtonBox(ConfigWindow)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Apply|QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout.addWidget(self.buttonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout,0,0,1,1)

        self.retranslateUi(ConfigWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ConfigWindow)
        ConfigWindow.setTabOrder(self.hostList,self.hostAddButton)
        ConfigWindow.setTabOrder(self.hostAddButton,self.hostEditButton)
        ConfigWindow.setTabOrder(self.hostEditButton,self.hostDeleteButton)
        ConfigWindow.setTabOrder(self.hostDeleteButton,self.buttonBox)
        ConfigWindow.setTabOrder(self.buttonBox,self.tabWidget)

    def retranslateUi(self, ConfigWindow):
        ConfigWindow.setWindowTitle(QtGui.QApplication.translate("ConfigWindow", "OVSDB Monitor Configuration", None, QtGui.QApplication.UnicodeUTF8))
        self.hostAddButton.setText(QtGui.QApplication.translate("ConfigWindow", "Add", None, QtGui.QApplication.UnicodeUTF8))
        self.hostEditButton.setText(QtGui.QApplication.translate("ConfigWindow", "Edit", None, QtGui.QApplication.UnicodeUTF8))
        self.hostDeleteButton.setText(QtGui.QApplication.translate("ConfigWindow", "Delete", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.hosts), QtGui.QApplication.translate("ConfigWindow", "Hosts", None, QtGui.QApplication.UnicodeUTF8))
        self.logTrafficCheckBox.setToolTip(QtGui.QApplication.translate("ConfigWindow", "Whether to log traffic exchanges in the log window", None, QtGui.QApplication.UnicodeUTF8))
        self.logTrafficCheckBox.setText(QtGui.QApplication.translate("ConfigWindow", "Log traffic", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.logging), QtGui.QApplication.translate("ConfigWindow", "Logging", None, QtGui.QApplication.UnicodeUTF8))
        self.truncateUuidsCheckBox.setToolTip(QtGui.QApplication.translate("ConfigWindow", "Replaces UUIDs with a shorter string of the first few characters.  The tooltip still contains the full value", None, QtGui.QApplication.UnicodeUTF8))
        self.truncateUuidsCheckBox.setText(QtGui.QApplication.translate("ConfigWindow", "Truncate UUIDs", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.view), QtGui.QApplication.translate("ConfigWindow", "View", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVEConfig import *
from OVEFetch import *
from OVELogger import *
from OVEUtil import *

from Ui_MainWindow import *

class OVECommonWindow:
    def __init__(self, app, loadIndex = None):
        self.app = app
        self.intervalTimerId = None        
        self.hostUuid = ''
        self.intervalChecked = True
        self.intervalSeconds = 5
        self.fetchSkip = 0
        self.currentRef = self.BASE_REF
                
        self.ui.setupUi(self)
        
        if loadIndex is not None:
            self.loadSettings(loadIndex)
        
        self.connect(self.ui.actionNew_DB_Window, QtCore.SIGNAL("triggered()"), self.xon_actionNew_DB_Window_triggered)
        self.connect(self.ui.actionNew_Flow_Window, QtCore.SIGNAL("triggered()"), self.xon_actionNew_Flow_Window_triggered)
        self.connect(self.ui.actionShow_Log, QtCore.SIGNAL("triggered()"), self.xon_actionShow_Log_triggered)
        self.connect(self.ui.actionPreferences, QtCore.SIGNAL("triggered()"), self.xon_actionPreferences_triggered)
        self.connect(self.ui.actionQuit, QtCore.SIGNAL("triggered()"), self.xon_actionQuit_triggered)
        self.connect(self.ui.fetchButton, QtCore.SIGNAL("clicked()"), self.xon_fetchButton_clicked)
        self.connect(self.ui.tabWidget, QtCore.SIGNAL("currentChanged(int)"), self.xon_tabWidget_currentChanged)
        self.connect(self.ui.hostComboBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.xon_hostComboBox_currentIndexChanged)
        self.connect(self.ui.intervalCheckBox, QtCore.SIGNAL("stateChanged(int)"), self.xon_intervalCheckBox_stateChanged)
        self.connect(self.ui.intervalSpinBox, QtCore.SIGNAL("valueChanged(int)"), self.xon_intervalSpinBox_valueChanged)
        self.connect(OVEConfig.Inst(), QtCore.SIGNAL("configUpdated()"), self.xon_configUpdated)
        
        self.updateHosts()
        self.updateInterval()
        self.updateIntervalState()
        self.updateTable()
        
    def xon_actionNew_DB_Window_triggered(self):
        self.app.newMainWindow()
 
    def xon_actionNew_Flow_Window_triggered(self):
        self.app.newFlowWindow()

    def xon_actionShow_Log_triggered(self):
        self.app.showLog(True)

    def xon_actionPreferences_triggered(self):
        self.app.showConfig(True)

    def xon_actionQuit_triggered(self):
        self.app.quit()
    
    def xon_tabWidget_currentChanged(self, value):
        self.updateTable()
        
    def xon_fetchButton_clicked(self):
        self.updateTable()
    
    def xon_configUpdated(self):
        self.updateHosts()
    
    def xon_hostComboBox_currentIndexChanged(self, index):
        if (index >= 0):
            itemData = self.ui.hostComboBox.itemData(index)
            self.hostUuid = str(itemData.toString())
            self.deleteCurrentTable()
            self.updateTable()
    
    def xon_intervalCheckBox_stateChanged(self, state):
        self.intervalChecked = (state == Qt.Checked)
        self.updateIntervalState()
    
    def xon_intervalSpinBox_valueChanged(self, value):
        self.intervalSeconds = value
        self.updateIntervalState()
    
    def updateIntervalState(self):
        if self.intervalTimerId is not None:
            self.killTimer(self.intervalTimerId)
        if self.intervalChecked:
            self.intervalTimerId = self.startTimer(1000*self.intervalSeconds)
    
    def updateHosts(self):
        currentHostUuid = self.hostUuid # self.hostUuid will change due to currentIndexChanged events as we rebuild the combo box
        self.hostUuid = ''
        self.ui.hostComboBox.clear()
        for i, host in enumerate(OVEConfig.Inst().hosts):
            self.ui.hostComboBox.addItem(host['address'], QVariant(host['uuid']))
            if host['uuid'] == currentHostUuid:
                # This is the currently selected host
                self.ui.hostComboBox.setCurrentIndex(i)
        if len(OVEConfig.Inst().hosts) == 0:
            self.ui.hostComboBox.addItem('(No hosts configured)', QVariant(''))
            
    def updateInterval(self):
        self.ui.intervalCheckBox.setChecked(self.intervalChecked)
        self.ui.intervalSpinBox.setValue(self.intervalSeconds)
 
    def handleFetchEvent(self, ref, values):
        OVELog('Unhandled FetchEvent')
 
    def handleFetchFailEvent(self, ref, message):
        OVELog('Unhandled FetchFailEvent')
 
    def setFetchSkip(self):
        # Call before sending a request via OVEFetch
        self.fetchSkip = 6
 
    def timerEvent(self, event):
        if event.timerId() == self.intervalTimerId:
            if self.fetchSkip > 0:
                self.statusBar().showMessage('Fetch stalled... resend in '+str(self.fetchSkip*self.intervalSeconds)+'s') 
                self.fetchSkip -= 1
                if self.fetchSkip == 0:
                    # Stall has timed out. The connection might have hung so reset.  Seems to happen with PySide only
                    OVEFetch.Inst(self.hostUuid).resetTransport()
            else:
                self.updateTable()
        else:
            QtGui.QMainWindow.timerEvent(self, event)
 
    def customEvent(self, event):
        if event.type() == OVEFetchEvent.TYPE:
            if isinstance(event, OVEFetchEvent):
                # The right way to get data
                ref = event.ref
                values = event.data
            else:
                # Workaround for PySide issue
                ref = OVEFetch.Inst(self.hostUuid).snoopRef(self)
                values = OVEFetch.Inst(self.hostUuid).snoopValues(self)
            try:
                if ref == self.currentRef:
                    self.fetchSkip = 0
                    self.currentRef += 1 # PySide workaround
                    self.handleFetchEvent(ref, values)
                else:
                    # If refs don't match this event relates to a request before the current one.  We've moved
                    # on since then, e.g. changed the table we've viewing, so ignore it
                    if OVEConfig.Inst().logTraffic:
                        OVELog('FetchEvent ref mismatch '+str(ref)+' != '+str(self.currentRef))
            except Exception, e:
                OVELog("Error during data handling: "+str(e))

        elif event.type() == OVEFetchFailEvent.TYPE:
            if isinstance(event, OVEFetchFailEvent):
                # The right way to get data
                ref = event.ref
                message = event.message
            else:
                # Workaround for PySide issue
                ref = OVEFetch.Inst(self.hostUuid).snoopRef(self)
                message = OVEFetch.Inst(self.hostUuid).snoopMessage(self)
            if message is not None:
                OVELog(message)
            if ref == self.currentRef:
                self.fetchSkip = 0
                self.currentRef += 1 # PySide workaround
                self.handleFetchFailEvent(ref, message)
            else:
                if OVEConfig.Inst().logTraffic:
                    OVELog('FetchFailEvent ref mismatch '+str(ref)+' != '+str(self.currentRef))
 
    def deleteCurrentTable(self):
        pass
 
    def saveSettings(self, index):
        key = self.LOAD_KEY+str(index)
        settings = QtCore.QSettings()
        settings.setValue(key+"/loadable", QVariant(True))
        settings.setValue(key+"/pos", QVariant(self.pos()))
        settings.setValue(key+"/size", QVariant(self.size()))
        settings.setValue(key+"/hostUuid", QVariant(self.hostUuid))
        settings.setValue(key+"/intervalChecked", QVariant(self.intervalChecked))
        settings.setValue(key+"/intervalSeconds", QVariant(self.intervalSeconds))

        return settings, key
    
    def loadSettings(self, index):
        key = self.LOAD_KEY+str(index)
        settings = QtCore.QSettings()
        pos = settings.value(key+"/pos", QVariant(QtCore.QPoint(200, 200))).toPoint()
        size = settings.value(key+"/size", QVariant(QtCore.QSize(400, 400))).toSize();

        self.hostUuid = str(settings.value(key+"/hostUuid", QVariant('Unloaded')).toString())
        self.intervalChecked = settings.value(key+"/intervalChecked", QVariant(True)).toBool()
        self.intervalSeconds = settings.value(key+"/intervalSeconds", QVariant(5)).toInt()[0]
        self.resize(size)
        self.move(pos)
        return settings, key

    @classmethod
    def terminateSettings(self, index):
        key = self.LOAD_KEY+str(index)
        settings = QtCore.QSettings()
        settings.setValue(key+"/loadable", QVariant(False))
        settings.sync()
        
    @classmethod
    def isLoadable(cls, index):
        key = cls.LOAD_KEY+str(index)
        settings = QtCore.QSettings()
        return settings.value(key+"/loadable", QVariant(False)).toBool()
        



# Copyright (c) 2011 Nicira, Inc.
# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *

from OVEConfig import *
import re

class OVEUtil:
    UUID_RE = re.compile(r'([a-f0-9]{8}-[a-f0-9]{2})[a-f0-9]{2}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}')
    
    @classmethod
    def paramToLongString(cls, param):
        if isinstance(param, (types.ListType, types.TupleType)) and len(param) > 1:
            text = str(param[1])
        else:
            text = str(param)

        return text.replace(', ', ',\n')
        
    @classmethod
    def paramToString(cls, param):
        if isinstance(param, (types.ListType, types.TupleType)) and len(param) > 1:
            text = str(param[1])
        else:
            text = str(param)
        if OVEConfig.Inst().truncateUuids:
            text = cls.UUID_RE.sub('\\1...', text)
            
        return text.replace(', ', ',\n')

    @classmethod
    def flowDecodeHeadings(self):
        return [
                'Type',
                'Proto',
                'Inport',
                'VLAN',
                'Source MAC',
                'Destination MAC',
                'Source IP',
                'Destination IP',
                'Src port',
                'Dest port',
                'Packet count',
                'Bytes',
                'Used',
                'Tos',
                'PCP',
                'Tunnel',
                'Actions',
                ]

    @classmethod
    def getFlowColumn(cls, name):
        lowerName = name.lower()
        for i, columnName in enumerate(cls.flowDecodeHeadings()):
            if lowerName == columnName.lower():
                return i
        return None

    ETHERTYPE_TRANS = {
        '05ff':'ESX probe',
        '0800':'IP',
        '0806':'ARP',
        '86dd':'IPv6',
        '88cc':'LLDP'
    }
                  
    ETHERPROTO_TRANS = {
        '1':'ICMP',
        '6':'TCP',
        '17':'UDP'
    }
    
    # Parsing of ovs-dpctl dump-flows output should be localised in this method and flowDecodeHeadings
    @classmethod
    def decodeFlows(cls, srcLines):
        retVal = []
        for line in srcLines.split('\n'):
            if line != '':
                fields = {}
                for name, val in re.findall(r'([a-zA-Z0-9_+]+)\(([^)]+)\)', line):
                    if '=' in val:
                        for setting in val.split(','):
                            k,v = setting.split('=')
                            fields['%s.%s' % (name, k)] = v
                    else:
                        fields[name] = val
                for setting in re.split(', ', line)[1:]:
                    if ':' in setting:
                        k,v = setting.split(':')
                        fields[k] = v

                tun_id = fields.get('tun_id', '')
                in_port = int(fields.get('in_port', 0))
                eth_src = fields.get('eth.src', '')
                eth_dst = fields.get('eth.dst', '')
                vlan_vid = int(fields.get('vlan.vid', 0))
                vlan_pcp = int(fields.get('vlan.pcp', 0))
                eth_type = fields.get('eth_type', '')
                ip_src = fields.get('ipv4.src', fields.get('ipv6.src', ''))
                ip_dst = fields.get('ipv4.dst', fields.get('ipv6.dst', ''))
                ip_proto = fields.get('ipv4.proto', fields.get('ipv6.proto', ''))
                ip_tos = fields.get('ipv4.tos', fields.get('ipv6.tos', ''))
                tp_src = fields.get('tcp.src', fields.get('udp.src', fields.get('arp.sip', fields.get('icmp.type', fields.get('icmpv6.type', '')))))
                tp_dst = fields.get('tcp.dst', fields.get('udp.dst', fields.get('arp.tip', fields.get('icmp.code', fields.get('icmpv6.code', '')))))

                packets = fields.get('packets', '')
                bytes = fields.get('bytes', '')
                actions = fields.get('actions', '')
                used = fields.get('used', '')

                # Order below needs to match that in flowDecodeHeadings
                retVal.append((eth_type, ip_proto, in_port, vlan_vid, eth_src, eth_dst, ip_src, ip_dst, tp_src, tp_dst, packets, bytes, used, ip_tos, vlan_pcp, tun_id, actions))
                    
        return retVal
        
    COLOURS = [Qt.black, Qt.darkBlue,  Qt.darkRed, Qt.darkGreen, Qt.darkMagenta, Qt.darkCyan, Qt.darkGray, Qt.darkYellow, Qt.blue, Qt.gray, Qt.magenta, Qt.red]
        
    @classmethod
    def intToColour(cls, value):
        return cls.COLOURS[value % len(cls.COLOURS)]



# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVEConfig import *
from OVEFetch import *
from OVELogger import *
from OVEUtil import *

from OVECommonWindow import *

from Ui_MainWindow import *

class OVEMainWindow(QtGui.QMainWindow, OVECommonWindow):
    LOAD_KEY = 'MainWindow/window'
    BASE_REF=100000
    
    def __init__(self, app, loadIndex = None):
        QtGui.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        OVECommonWindow.__init__(self, app, loadIndex)
    
    def xon_tabWidget_currentChanged(self, value):
        self.deleteCurrentTable()
        OVECommonWindow.xon_tabWidget_currentChanged(self, value)

    def updateTable(self):
        if self.hostUuid == '':
            self.setWindowTitle('OVS Database')
            self.deleteCurrentTable()
            self.statusBar().showMessage('No host selected.  Choose File->Preferences to add a host')
        else:
            config = OVEConfig.Inst().hostFromUuid(self.hostUuid)
            self.setWindowTitle('OVS Database - '+config.get('address', ''))
            self.invalidateCurrentTable('Fetching data...')
            tabName = self.ui.tabWidget.currentWidget().objectName()
            try:
                self.setFetchSkip()
                self.currentRef += 1
                OVEFetch.Inst(self.hostUuid).getTable(self, tabName, self.currentRef)
            except Exception, e:
                OVELog("Error fetching data: "+str(e))
                self.invalidateCurrentTable(str(e))

    def timerEvent(self, event):
        OVECommonWindow.timerEvent(self, event)

    def customEvent(self, event):
        OVECommonWindow.customEvent(self, event)

    def handleFetchEvent(self, ref, values):
        tabName = self.ui.tabWidget.currentWidget().objectName()
        self.structToTable(getattr(self.ui, str(tabName)+'Table'), values)
 
    def handleFetchFailEvent(self, ref, message):
        self.invalidateCurrentTable(str(message))

    def structToTable(self, table, values):
        
        table.setUpdatesEnabled(False)
        table.setSortingEnabled(False)
        
        for result in values:
            rowNum = 0
            table.setRowCount(len(result['rows']))
            for row in result['rows']:
                table.setColumnCount(len(row))
                colNum=0
                for k in sorted(row.keys()):
                    v = row[k]
                    headerItem = QtGui.QTableWidgetItem(k)
                    table.setHorizontalHeaderItem(colNum, headerItem)
                    text = OVEUtil.paramToString(v)
                    item = QtGui.QTableWidgetItem(text)
                    longText = OVEUtil.paramToLongString(v)
                    item.setToolTip(longText)

                    table.setItem(rowNum, colNum, item)
                    colNum+=1

                rowNum+=1
                
        for i in range(0, table.columnCount()):
            table.resizeColumnToContents(i)
        for i in range(0, table.rowCount()):
            table.resizeRowToContents(i)
        
        # table.setSortingEnabled(True)
        table.setUpdatesEnabled(True)
        
        message = 'Updated at '+str(QtCore.QTime.currentTime().toString())
        if table.rowCount() == 0:
            message += ' - Table is empty'
        self.statusBar().showMessage(message)

    def invalidateCurrentTable(self, message):
        tabName = self.ui.tabWidget.currentWidget().objectName()
        self.invalidateTable(getattr(self.ui, str(tabName)+'Table'), message)

    def invalidateTable(self, table, message):
        table.setUpdatesEnabled(False)
        table.setSortingEnabled(False)
        
        for rowNum in range(0, table.rowCount()):
            for colNum in range(0, table.columnCount()):
                item = table.takeItem(rowNum, colNum)
                if item is not None:
                    item.setForeground(Qt.darkGray)
                    table.setItem(rowNum, colNum, item)
        self.statusBar().showMessage(message)
        # table.setSortingEnabled(True)
        table.setUpdatesEnabled(True)

    def deleteCurrentTable(self):
        tabName = self.ui.tabWidget.currentWidget().objectName()
        self.deleteTable(getattr(self.ui, str(tabName)+'Table'))

    def deleteTable(self, table):
        table.clear()
        table.setRowCount(0)
        table.setColumnCount(0)
        
    def saveSettings(self, index):
        settings = OVECommonWindow.saveSettings(self, index)
    
    def loadSettings(self, index):
        settings = OVECommonWindow.loadSettings(self, index)



# Copyright (c) 2011 Nicira, Inc.
# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os, re, struct, sys, time, types, uuid
from copy import deepcopy
from pprint import pprint

# Set globalForcePySide to True to use PySide instead of PyQt if both are installed
globalForcePySide = False

try:
    import ovs.json
except Exception, e:
    print('+++ OVS JSON module is required\n')
    raise

try:
    if globalForcePySide:
        print('Forcing use of PySide')
        raise Exception()
    from PyQt4.QtCore import Qt, QVariant
    from PyQt4 import QtCore, QtGui
except:
    try:
        from PySide.QtCore import Qt, QVariant
        from PySide import QtCore, QtGui
    except Exception, e:
        print('+++ This application requires either PyQt4 or PySide\n')
        raise




# Copyright (c) 2009, 2010 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time


def msec():
    """Returns the current time, as the amount of time since the epoch, in
    milliseconds, as a float."""
    return time.time() * 1000.0


def postfork():
    # Just a stub for now
    pass



# Copyright (c) 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import signal
import sys

import ovs.daemon
import ovs.unixctl
import ovs.unixctl.client
import ovs.util
import ovs.vlog


def connect_to_target(target):
    error, str_result = ovs.unixctl.socket_name_from_target(target)
    if error:
        ovs.util.ovs_fatal(error, str_result)
    else:
        socket_name = str_result

    error, client = ovs.unixctl.client.UnixctlClient.create(socket_name)
    if error:
        ovs.util.ovs_fatal(error, "cannot connect to \"%s\"" % socket_name)

    return client


def main():
    parser = argparse.ArgumentParser(description="Python Implementation of"
                                     " ovs-appctl.")
    parser.add_argument("-t", "--target", default="ovs-vswitchd",
                        help="pidfile or socket to contact")

    parser.add_argument("command", metavar="COMMAND",
                        help="Command to run.")
    parser.add_argument("argv", metavar="ARG", nargs="*",
                        help="Arguments to the command.")
    parser.add_argument("-T", "--timeout", metavar="SECS",
                        help="wait at most SECS seconds for a response")
    args = parser.parse_args()

    if args.timeout:
        signal.alarm(int(args.timeout))

    ovs.vlog.Vlog.init()
    target = args.target
    client = connect_to_target(target)
    err_no, error, result = client.transact(args.command, args.argv)
    client.close()

    if err_no:
        ovs.util.ovs_fatal(err_no, "%s: transaction error" % target)
    elif error is not None:
        sys.stderr.write(error)
        ovs.util.ovs_error(0, "%s: server returned an error" % target)
        sys.exit(2)
    else:
        assert result is not None
        sys.stdout.write(result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../ovsdb/ovsdbmonitor/MainWindow.ui'
#
# Created: Mon May 17 16:23:47 2010
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!


try:
    from OVEStandard import globalForcePySide
    if globalForcePySide:
        raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.Bridge = QtGui.QWidget()
        self.Bridge.setObjectName("Bridge")
        self.gridLayout_2 = QtGui.QGridLayout(self.Bridge)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.BridgeTable = QtGui.QTableWidget(self.Bridge)
        self.BridgeTable.setObjectName("BridgeTable")
        self.BridgeTable.setColumnCount(0)
        self.BridgeTable.setRowCount(0)
        self.gridLayout_2.addWidget(self.BridgeTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Bridge, "")
        self.Controller = QtGui.QWidget()
        self.Controller.setObjectName("Controller")
        self.gridLayout_3 = QtGui.QGridLayout(self.Controller)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.ControllerTable = QtGui.QTableWidget(self.Controller)
        self.ControllerTable.setObjectName("ControllerTable")
        self.ControllerTable.setColumnCount(0)
        self.ControllerTable.setRowCount(0)
        self.gridLayout_3.addWidget(self.ControllerTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Controller, "")
        self.Interface = QtGui.QWidget()
        self.Interface.setObjectName("Interface")
        self.gridLayout_4 = QtGui.QGridLayout(self.Interface)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.InterfaceTable = QtGui.QTableWidget(self.Interface)
        self.InterfaceTable.setObjectName("InterfaceTable")
        self.InterfaceTable.setColumnCount(0)
        self.InterfaceTable.setRowCount(0)
        self.gridLayout_4.addWidget(self.InterfaceTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Interface, "")
        self.Mirror = QtGui.QWidget()
        self.Mirror.setObjectName("Mirror")
        self.gridLayout_5 = QtGui.QGridLayout(self.Mirror)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.MirrorTable = QtGui.QTableWidget(self.Mirror)
        self.MirrorTable.setObjectName("MirrorTable")
        self.MirrorTable.setColumnCount(0)
        self.MirrorTable.setRowCount(0)
        self.gridLayout_5.addWidget(self.MirrorTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Mirror, "")
        self.NetFlow = QtGui.QWidget()
        self.NetFlow.setObjectName("NetFlow")
        self.gridLayout_6 = QtGui.QGridLayout(self.NetFlow)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.NetFlowTable = QtGui.QTableWidget(self.NetFlow)
        self.NetFlowTable.setObjectName("NetFlowTable")
        self.NetFlowTable.setColumnCount(0)
        self.NetFlowTable.setRowCount(0)
        self.gridLayout_6.addWidget(self.NetFlowTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.NetFlow, "")
        self.Open_vSwitch = QtGui.QWidget()
        self.Open_vSwitch.setObjectName("Open_vSwitch")
        self.gridLayout_7 = QtGui.QGridLayout(self.Open_vSwitch)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.Open_vSwitchTable = QtGui.QTableWidget(self.Open_vSwitch)
        self.Open_vSwitchTable.setObjectName("Open_vSwitchTable")
        self.Open_vSwitchTable.setColumnCount(0)
        self.Open_vSwitchTable.setRowCount(0)
        self.gridLayout_7.addWidget(self.Open_vSwitchTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Open_vSwitch, "")
        self.Port = QtGui.QWidget()
        self.Port.setObjectName("Port")
        self.gridLayout_8 = QtGui.QGridLayout(self.Port)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.PortTable = QtGui.QTableWidget(self.Port)
        self.PortTable.setObjectName("PortTable")
        self.PortTable.setColumnCount(0)
        self.PortTable.setRowCount(0)
        self.gridLayout_8.addWidget(self.PortTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Port, "")
        self.QoS = QtGui.QWidget()
        self.QoS.setObjectName("QoS")
        self.gridLayout_10 = QtGui.QGridLayout(self.QoS)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.QoSTable = QtGui.QTableWidget(self.QoS)
        self.QoSTable.setObjectName("QoSTable")
        self.QoSTable.setColumnCount(0)
        self.QoSTable.setRowCount(0)
        self.gridLayout_10.addWidget(self.QoSTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.QoS, "")
        self.Queue = QtGui.QWidget()
        self.Queue.setObjectName("Queue")
        self.gridLayout_11 = QtGui.QGridLayout(self.Queue)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.QueueTable = QtGui.QTableWidget(self.Queue)
        self.QueueTable.setObjectName("QueueTable")
        self.QueueTable.setColumnCount(0)
        self.QueueTable.setRowCount(0)
        self.gridLayout_11.addWidget(self.QueueTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.Queue, "")
        self.sFlow = QtGui.QWidget()
        self.sFlow.setObjectName("sFlow")
        self.gridLayout_9 = QtGui.QGridLayout(self.sFlow)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.sFlowTable = QtGui.QTableWidget(self.sFlow)
        self.sFlowTable.setObjectName("sFlowTable")
        self.sFlowTable.setColumnCount(0)
        self.sFlowTable.setRowCount(0)
        self.gridLayout_9.addWidget(self.sFlowTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.sFlow, "")
        self.SSL = QtGui.QWidget()
        self.SSL.setObjectName("SSL")
        self.gridLayout_101 = QtGui.QGridLayout(self.SSL)
        self.gridLayout_101.setObjectName("gridLayout_101")
        self.SSLTable = QtGui.QTableWidget(self.SSL)
        self.SSLTable.setObjectName("SSLTable")
        self.SSLTable.setColumnCount(0)
        self.SSLTable.setRowCount(0)
        self.gridLayout_101.addWidget(self.SSLTable, 0, 0, 1, 1)
        self.tabWidget.addTab(self.SSL, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.hostLabel = QtGui.QLabel(self.centralwidget)
        self.hostLabel.setObjectName("hostLabel")
        self.horizontalLayout.addWidget(self.hostLabel)
        self.hostComboBox = QtGui.QComboBox(self.centralwidget)
        self.hostComboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.hostComboBox.setObjectName("hostComboBox")
        self.horizontalLayout.addWidget(self.hostComboBox)
        self.intervalCheckBox = QtGui.QCheckBox(self.centralwidget)
        self.intervalCheckBox.setObjectName("intervalCheckBox")
        self.horizontalLayout.addWidget(self.intervalCheckBox)
        self.intervalSpinBox = QtGui.QSpinBox(self.centralwidget)
        self.intervalSpinBox.setMinimum(1)
        self.intervalSpinBox.setMaximum(1000000)
        self.intervalSpinBox.setObjectName("intervalSpinBox")
        self.horizontalLayout.addWidget(self.intervalSpinBox)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.fetchButton = QtGui.QPushButton(self.centralwidget)
        self.fetchButton.setObjectName("fetchButton")
        self.horizontalLayout.addWidget(self.fetchButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 28))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionShow_Log = QtGui.QAction(MainWindow)
        self.actionShow_Log.setObjectName("actionShow_Log")
        self.actionNew_DB_Window = QtGui.QAction(MainWindow)
        self.actionNew_DB_Window.setObjectName("actionNew_DB_Window")
        self.actionPreferences = QtGui.QAction(MainWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionNew_Flow_Window = QtGui.QAction(MainWindow)
        self.actionNew_Flow_Window.setObjectName("actionNew_Flow_Window")
        self.menuFile.addAction(self.actionNew_DB_Window)
        self.menuFile.addAction(self.actionNew_Flow_Window)
        self.menuFile.addAction(self.actionShow_Log)
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.hostLabel.setBuddy(self.hostComboBox)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "OVSDB Monitor", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Bridge), QtGui.QApplication.translate("MainWindow", "Bridge", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Controller), QtGui.QApplication.translate("MainWindow", "Controller", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Interface), QtGui.QApplication.translate("MainWindow", "Interface", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Mirror), QtGui.QApplication.translate("MainWindow", "Mirror", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.NetFlow), QtGui.QApplication.translate("MainWindow", "NetFlow", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Open_vSwitch), QtGui.QApplication.translate("MainWindow", "Open_vSwitch", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Port), QtGui.QApplication.translate("MainWindow", "Port", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.QoS), QtGui.QApplication.translate("MainWindow", "QoS", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.Queue), QtGui.QApplication.translate("MainWindow", "Queue", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.sFlow), QtGui.QApplication.translate("MainWindow", "sFlow", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.SSL), QtGui.QApplication.translate("MainWindow", "SSL", None, QtGui.QApplication.UnicodeUTF8))
        self.hostLabel.setText(QtGui.QApplication.translate("MainWindow", "Host", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalCheckBox.setText(QtGui.QApplication.translate("MainWindow", "Auto-refetch every", None, QtGui.QApplication.UnicodeUTF8))
        self.intervalSpinBox.setSuffix(QtGui.QApplication.translate("MainWindow", "s", None, QtGui.QApplication.UnicodeUTF8))
        self.fetchButton.setText(QtGui.QApplication.translate("MainWindow", "Refetch", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow_Log.setText(QtGui.QApplication.translate("MainWindow", "Show Log", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_DB_Window.setText(QtGui.QApplication.translate("MainWindow", "New DB Window", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPreferences.setText(QtGui.QApplication.translate("MainWindow", "Preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("MainWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Flow_Window.setText(QtGui.QApplication.translate("MainWindow", "New Flow Window", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from OVEStandard import *
from OVEConfig import *
from OVEFetch import *

from OVEConfigWindow import *
from OVEFlowWindow import *
from OVELogWindow import *
from OVEMainWindow import *

class OVEApp:
    def __init__(self):
        self.app = globalApp
        self.app.setOrganizationName("Citrix_Systems_Inc")
        self.app.setOrganizationDomain("citrix.com")
        self.app.setApplicationName("ovsdbmonitor")
        self.mainWindows = []
        self.flowWindows = []
        self.configWindow = None

    def enter(self):
        if len(OVEConfig.Inst().hosts) < 1:
            self.showConfig(True)
            QtGui.QMessageBox.information(
                    None, "OVSDB Monitor",
                    "This application browses openvswitch databases on remote hosts.  Please add one or more openvswitch hosts to continue")
        self.loadMainWindows()
        self.loadFlowWindows()
        if len(self.mainWindows) == 0 and len(self.flowWindows) == 0:
            self.newMainWindow()
        self.newLogWindow()
        # Reactor must be started after the event loop is running, so use a zero timeout
        QtCore.QTimer.singleShot(0, OVEFetch.startReactor)
        OVELog("Application started")
        retCode = self.app.exec_()
        index = 0
        for mainWindow in self.mainWindows:
            if mainWindow.isVisible():
                mainWindow.saveSettings(index)
                index += 1 # Indent intentional
        OVEMainWindow.terminateSettings(index)
        index = 0
        for flowWindow in self.flowWindows:
            if flowWindow.isVisible():
                flowWindow.saveSettings(index)
                index += 1 # Indent intentional            
        OVEFlowWindow.terminateSettings(index)
        self.logWindow.saveSettings()
    
    def quit(self):
        self.app.quit()
    
    def showLog(self, value):
        if value:
            self.logWindow.hide()
            self.logWindow.show()
        else:
            self.logWindow.hide()

    def showConfig(self, value):
        if value:
            del self.configWindow
            self.configWindow = OVEConfigWindow(self)
            self.configWindow.show()
        else:
            self.configWindow.hide()

    def newMainWindow(self, loadIndex = None):
        self.mainWindows.append(OVEMainWindow(self, loadIndex))
        self.mainWindows[-1].show()

    def newFlowWindow(self, loadIndex = None):
        self.flowWindows.append(OVEFlowWindow(self, loadIndex))
        self.flowWindows[-1].show()

    def newLogWindow(self):
        self.logWindow = OVELogWindow(self)

    def loadMainWindows(self):
        for loadIndex in range(0, 100):
            if OVEMainWindow.isLoadable(loadIndex):
                self.newMainWindow(loadIndex)
            else:
                break

    def loadFlowWindows(self):
        for loadIndex in range(0, 100):
            if OVEFlowWindow.isLoadable(loadIndex):
                self.newFlowWindow(loadIndex)
            else:
                break



# Copyright (c) 2011 Nicira, Inc.
# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVEConfig import *
from OVELogger import *
import ovs.json

# This sequence installs the qt4reactor before twisted gets a chance to install its reactor
import qt4reactor
globalApp = QtGui.QApplication([])
qt4reactor.install()

try:
    from twisted.conch.ssh import transport, userauth, connection, common, keys, channel
    from twisted.internet import defer, protocol, reactor
    from twisted.application import reactors
except Exception, e:
    print('+++ Python Twisted Conch module is required\n')
    raise

class OVEFetchUserAuth(userauth.SSHUserAuthClient):
    def __init__(self, fetch, *params):
        userauth.SSHUserAuthClient.__init__(self, *params)
        self.fetch = fetch
        self.authFails = 0
    
    def getPassword(self):
        return defer.succeed(self.fetch.config()['password'])

    def ssh_USERAUTH_FAILURE(self, packet):
        if self.authFails > 0: # We normally get one so ignore.  Real failures send these repeatedly
            OVELog('Authentication failure for '+self.fetch.config()['address'])
        self.authFails += 1
        userauth.SSHUserAuthClient.ssh_USERAUTH_FAILURE(self, packet)

class OVEFetchConnection(connection.SSHConnection, QtCore.QObject):
    def __init__(self, fetch, *params):
        connection.SSHConnection.__init__(self, *params)
        QtCore.QObject.__init__(self)
        self.fetch = fetch
        self._channel = None
        self._oldChannels = []
        
    def serviceStarted(self):
        self.emit(QtCore.SIGNAL('connectionService(QObject)'), self)

    def serviceStopped(self):
        self.emit(QtCore.SIGNAL('connectionService(QObject)'), None)

    def execCommand(self, requester, ref, command, commandType):
        if self._channel is not None:
            # Don't delete old channels immediately in case they're e.g. going to time out with a failure
            self._oldChannels.append(self._channel)
            if len(self._oldChannels) > 90:
                # For 30 second timeouts at 1 second refresh interval and three windows open on a single host, need 90 channels
                del self._oldChannels[1]
        self._channel = OVECommandChannel(self.fetch, requester, ref, command, commandType, 2**16, 2**15, self)
        self.openChannel(self._channel)

    def connectionLost(self, reason):
        if self._channel is not None:
            self._channel.connectionLost(reason)

class OVEFetchTransport(transport.SSHClientTransport, QtCore.QObject):
    def __init__(self, fetch, *params):
        # There is no __init__ method for this class
        # transport.SSHClientTransport.__init__(self, *params)
        
        QtCore.QObject.__init__(self)
        self.fetch = fetch
        self._connection = None
        self.connect(self, QtCore.SIGNAL('channelFailure(QObject, int, QString, QString, QString)'), self.fetch.xon_channelFailure)
        
    def verifyHostKey(self, hostKey, fingerprint):
        return defer.succeed(1)

    def connectionSecure(self):
        self._connection = OVEFetchConnection(self.fetch)
        QtCore.QObject.connect(self._connection, QtCore.SIGNAL('connectionService(QObject)'), self.fetch.xon_connectionService)
        self.requestService(
            OVEFetchUserAuth(self.fetch, self.fetch.config().get('username', 'root'),
                self._connection))

    def connectionLost(self, reason):
        if self._connection is not None:
            self._connection.connectionLost(reason)

class OVEFetchWrapper:
    def __init__(self, contents):
        self.contents = contents

class OVECommandChannel(channel.SSHChannel, QtCore.QObject):
    name = 'session'
    MSEC_TIMEOUT=10000
    STATUS_CONNECTION_LOST = 100001
    STATUS_TIMEOUT = 100002
    END_MARKER='END-MARKER'
    END_MARKER_RE=re.compile(r'^END-MARKER$', re.MULTILINE)
    
    def __init__(self, fetch, requester, ref, command, commandType, *params):
        channel.SSHChannel.__init__(self, *params)
        QtCore.QObject.__init__(self)        
        self.fetch = fetch
        self.requester = requester
        self.ref = ref
        self.command = command
        self.commandType= commandType
        self._data = ''
        self._extData = ''
        self._jsonValues = None
        self._timerId = None
        self._status = None
        self.connect(self, QtCore.SIGNAL('channelData(QObject, int, QString)'), self.fetch.xon_channelData)
        self.connect(self, QtCore.SIGNAL('channelExtData(QObject, int, QString)'), self.fetch.xon_channelExtData)
        self.connect(self, QtCore.SIGNAL('channelSuccess(QObject, int, QString, QString, QVariant)'), self.fetch.xon_channelSuccess)
        self.connect(self, QtCore.SIGNAL('channelFailure(QObject, int, QString, QString, QString)'), self.fetch.xon_channelFailure)
        
    def openFailed(self, reason):
        if self._timerId is not None:
            self.killTimer(self._timerId)
        self.emit(QtCore.SIGNAL('channelFailure(QObject, int, QString, QString, QString)'), self.requester, self.ref,
            'Open failed:'+str(reason), '', '')

    def channelOpen(self, ignoredData):
        try:
            nsCommand = common.NS(str(self.command))
            self._timerId = self.startTimer(self.MSEC_TIMEOUT)
            self.conn.sendRequest(self, 'exec', nsCommand, wantReply=1)
        except Exception, e:
            self.emit(QtCore.SIGNAL('channelFailure(QObject, int, QString, QString, QString)'), self.requester, self.ref,
                'Open failed:'+str(e), self._data, self._extData)
            
    def dataReceived(self, data):
        self._data += data
        if OVEConfig.Inst().logTraffic:
            self.emit(QtCore.SIGNAL('channelData(QObject, int, QString)'), self.requester, self.ref, data)
        self.testIfDone()
        
    def extDataReceived(self, extData):
        self._extData += extData
        if OVEConfig.Inst().logTraffic:
            self.emit(QtCore.SIGNAL('channelExtData(QObject, int, QString)'), self.requester, self.ref, extData)

    def request_exit_status(self, data):
        # We can get the exit status before the data, so delay calling sendResult until we get both
        self._status = struct.unpack('>L', data)[0]
        self.testIfDone()
        
    def testIfDone(self):
        if self._status is not None:
            if self._status != 0:
                self.sendResult() # Failed, so send what we have
            elif len(self._data) > 0:
                # Status == success and we have some data
                if self.commandType == 'JSON':
                    try:
                        # Decode the JSON data, to confirm that we have all of the data
                        self._jsonValues = ovs.json.from_string(str(self._data)) # FIXME: Should handle unicode
                        self.sendResult()
                    except:
                        pass # Wait for more data
                elif self.commandType == 'framed':
                    match = self.END_MARKER_RE.search(self._data)
                    if match:
                        self._data = self._data[:match.start()] # Remove end marker
                        self.sendResult()
                else:
                    OVELog('Bad command type')

    def sendResult(self):
        if self._timerId is not None:
            self.killTimer(self._timerId)
        if self.commandType == 'JSON' and self._status == 0 and self._jsonValues is not None:
            self.emit(QtCore.SIGNAL('channelSuccess(QObject, int, QString, QString, QVariant)'), self.requester, self.ref, self._data, self._extData, QVariant(OVEFetchWrapper(self._jsonValues)))
        elif self.commandType != 'JSON' and self._status == 0:
            self.emit(QtCore.SIGNAL('channelSuccess(QObject, int, QString, QString, QVariant)'), self.requester, self.ref, self._data, self._extData, QVariant(None))
        else:
            self.emit(QtCore.SIGNAL('channelFailure(QObject, int, QString, QString, QString)'), self.requester, self.ref, 'Remote command failed (rc='+str(self._status)+')', self._data, self._extData)
        if self._status != self.STATUS_CONNECTION_LOST:
            try:
                self.loseConnection()
            except Exception, e:
                OVELog('OVECommandChannel.sendResult loseConnection error: '+str(e))

    def connectionLost(self, reason):
        self._extData += '+++ Connection lost'
        self._status = self.STATUS_CONNECTION_LOST
        self.sendResult()

    def timerEvent(self, event):
        if event.timerId() == self._timerId:
            self._extData += '+++ Timeout'
            self._status = self.STATUS_TIMEOUT
            self.sendResult()
        else:
            QtCore.QObject.timerEvent(self, event)

class OVEFetchEvent(QtCore.QEvent):
    TYPE = QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
    def __init__(self, ref, data):
        QtCore.QEvent.__init__(self, self.TYPE)
        self.ref = ref
        self.data = data

class OVEFetchFailEvent(QtCore.QEvent):
    TYPE = QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
    def __init__(self, ref, message):
        QtCore.QEvent.__init__(self, self.TYPE)
        self.ref = ref
        self.message = str(message)

class OVEFetch(QtCore.QObject):
    instances = {}
    SEC_TIMEOUT = 10.0
    
    def __init__(self, uuid):
        QtCore.QObject.__init__(self)
        self._hostUuid = uuid
        self._config = None
        self._transport = None
        self._connection = None
        self._commandQueue = []
        self._timerRef = 0
        self.refs = {}
        self.messages = {}
        self.values = {}
        self.connect(OVEConfig.Inst(), QtCore.SIGNAL("configUpdated()"), self.xon_configUpdated)
        
    @classmethod
    def Inst(cls, uuid):
        if uuid not in cls.instances:
            cls.instances[uuid] = OVEFetch(uuid)
        return cls.instances[uuid]

    @classmethod
    def startReactor(cls):
        reactor.runReturn()

    def xon_configUpdated(self):
        self._config = None
        self.resetTransport()
        
    def xon_connectionService(self, connection):
        self._connection = connection
        if self._connection is not None:
            OVELog('SSH connection to '+self.config()['address'] +' established')
            for command in self._commandQueue:
                # OVELog('Unqueueing '+str(command))
                self.execCommand2(*command)
            self._commandQueue = []

    def xon_channelData(self, requester, ref, data):
        if OVEConfig.Inst().logTraffic:
            OVELog('Channel data received: '+str(data))

    def xon_channelExtData(self, requester, ref, data):
        if OVEConfig.Inst().logTraffic:
            OVELog('+++ Channel extData (stderr) received: '+str(data))

    def xon_channelFailure(self, requester, ref, message, data, extData):
        if OVEConfig.Inst().logTraffic:
            OVELog('+++ Channel failure: '+str(message))
            OVELog("Closing SSH session due to failure")

        errMessage = message
        if len(data) > 0:
            errMessage += '\n+++ Failed command output: '+data
        if len(extData) > 0:
            errMessage += '\n+++ Failed command output (stderr): '+extData

        self.refs[requester] = ref # For PySide workaround
        self.messages[requester] = errMessage # For PySide workaround
        event = OVEFetchFailEvent(ref, errMessage)
        QtCore.QCoreApplication.postEvent(requester, event)
        self.resetTransport()
        
    def xon_channelSuccess(self, requester, ref, data, extData, jsonValueVariant):
        jsonValues = jsonValueVariant.toPyObject()
        if OVEConfig.Inst().logTraffic:
            OVELog('--- Channel success')
        try:
            if jsonValues is not None:
                values = jsonValues.contents
            else:
                values = str(data)

            self.refs[requester] = ref # For PySide workaround
            self.values[requester] = values # For PySide workaround
            event = OVEFetchEvent(ref, values)
            QtCore.QCoreApplication.postEvent(requester, event)
        except Exception, e:
            message = ('+++ Failed to decode JSON reply: '+str(e))
            if len(data) > 0: message += "\n++++++ Data (stdout): "+str(data)
            if len(extData) > 0: message += '\n++++++ Error (stderr): '+str(extData)
            self.refs[requester] = ref # For PySide workaround
            self.messages[requester] = message # For PySide workaround
            event = OVEFetchFailEvent(ref, message)
            QtCore.QCoreApplication.postEvent(requester, event)

    # Use for workaround only
    def snoopRef(self, requester):
        return self.refs.get(requester, None)

    # Use for workaround only
    def snoopValues(self, requester):
        return self.values.get(requester, None)

    # Use for workaround only
    def snoopMessage(self, requester):
        return self.messages.get(requester, None)

    def config(self):
        if self._config is None:
            self._config = OVEConfig.Inst().hostFromUuid(self._hostUuid)

        return self._config
    
    def resetTransport(self):
        if OVEConfig.Inst().logTraffic:
            OVELog('Transport reset for '+self.config()['address'])
        del self._connection
        del self._transport
        self._connection = None
        self._transport = None
        
    def transportErrback(self, failure, requester, ref, address):
        self._timerRef += 1 # Prevent timeout handling
        self.resetTransport()
        message = 'Failure connecting to '+address+': '+failure.getErrorMessage()
        self.refs[requester] = ref # For PySide workaround
        self.messages[requester] = message # For PySide workaround
        event = OVEFetchFailEvent(ref, message)
        QtCore.QCoreApplication.postEvent(requester, event)        
        
    def transportTimeout(self, timerRef, requester, ref, address):
        if self._timerRef == timerRef and self._transport is not None and self._connection is None:
            message = 'Connection attempt to ' +address+' timed out'
            self.refs[requester] = ref # For PySide workaround
            self.messages[requester] = message # For PySide workaround
            event = OVEFetchFailEvent(ref, message)
            QtCore.QCoreApplication.postEvent(requester, event)        
            self.resetTransport()

    def execCommand(self, requester, ref, command, commandType):
        if OVEConfig.Inst().logTraffic:
            hostName = (self.config() or {}).get('address', '<Address not set>')
            OVELog(str(QtCore.QTime.currentTime().toString())+' '+hostName+': Executing '+command)
        if self._transport is None:
            self._connection = None
            self._commandQueue.append((requester, ref, command, commandType))
            config = self.config()
            creator = protocol.ClientCreator(reactor, OVEFetchTransport, self)
            self._transport = creator.connectTCP(config['address'], config.get('port', 22), timeout = self.SEC_TIMEOUT)
            self._transport.addErrback(self.transportErrback, requester, ref, config['address'])
            self._timerRef += 1
            # Set this timer slightly longer than the twisted.conch timeout, as transportErrback can cancel
            # the timeout and prevent double handling
            # lambda timerRef = self._timerRef: takes a copy of self._timerRef
            QtCore.QTimer.singleShot(int((1+self.SEC_TIMEOUT) * 1000), lambda timerRef = self._timerRef: self.transportTimeout(timerRef, requester, ref, config['address']))
        else:
            self.execCommand2(requester, ref, command, commandType)

    def execCommand2(self, requester, ref, command, commandType):
        if self._connection is None:
            self._commandQueue.append((requester, ref, command, commandType))
        else:
            self._connection.execCommand(requester, ref, command, commandType)

    def getTable(self, requester, tableName, ref = QtCore.QObject()):
        command = '/usr/bin/ovsdb-client transact '+self.config()['connectTarget']+' \'["Open_vSwitch", {"op":"select","table":"'+tableName+'", "where":[]}]\''

        self.execCommand(requester, ref, command, 'JSON')
        
    def execCommandFramed(self, requester, ref, command):
        self.execCommand(requester, ref, command + ' && echo ' + OVECommandChannel.END_MARKER, 'framed')



# Copyright (c) 2010, 2011, 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import errno
import os
import socket

import ovs.poller
import ovs.socket_util
import ovs.vlog

vlog = ovs.vlog.Vlog("stream")


def stream_or_pstream_needs_probes(name):
    """ 1 if the stream or pstream specified by 'name' needs periodic probes to
    verify connectivity.  For [p]streams which need probes, it can take a long
    time to notice the connection was dropped.  Returns 0 if probes aren't
    needed, and -1 if 'name' is invalid"""

    if PassiveStream.is_valid_name(name) or Stream.is_valid_name(name):
        # Only unix and punix are supported currently.
        return 0
    else:
        return -1


class Stream(object):
    """Bidirectional byte stream.  Currently only Unix domain sockets
    are implemented."""

    # States.
    __S_CONNECTING = 0
    __S_CONNECTED = 1
    __S_DISCONNECTED = 2

    # Kinds of events that one might wait for.
    W_CONNECT = 0               # Connect complete (success or failure).
    W_RECV = 1                  # Data received.
    W_SEND = 2                  # Send buffer room available.

    _SOCKET_METHODS = {}

    @staticmethod
    def register_method(method, cls):
        Stream._SOCKET_METHODS[method + ":"] = cls

    @staticmethod
    def _find_method(name):
        for method, cls in Stream._SOCKET_METHODS.items():
            if name.startswith(method):
                return cls
        return None

    @staticmethod
    def is_valid_name(name):
        """Returns True if 'name' is a stream name in the form "TYPE:ARGS" and
        TYPE is a supported stream type (currently only "unix:" and "tcp:"),
        otherwise False."""
        return bool(Stream._find_method(name))

    def __init__(self, socket, name, status):
        self.socket = socket
        self.name = name
        if status == errno.EAGAIN:
            self.state = Stream.__S_CONNECTING
        elif status == 0:
            self.state = Stream.__S_CONNECTED
        else:
            self.state = Stream.__S_DISCONNECTED

        self.error = 0

    # Default value of dscp bits for connection between controller and manager.
    # Value of IPTOS_PREC_INTERNETCONTROL = 0xc0 which is defined
    # in <netinet/ip.h> is used.
    IPTOS_PREC_INTERNETCONTROL = 0xc0
    DSCP_DEFAULT = IPTOS_PREC_INTERNETCONTROL >> 2

    @staticmethod
    def open(name, dscp=DSCP_DEFAULT):
        """Attempts to connect a stream to a remote peer.  'name' is a
        connection name in the form "TYPE:ARGS", where TYPE is an active stream
        class's name and ARGS are stream class-specific.  Currently the only
        supported TYPEs are "unix" and "tcp".

        Returns (error, stream): on success 'error' is 0 and 'stream' is the
        new Stream, on failure 'error' is a positive errno value and 'stream'
        is None.

        Never returns errno.EAGAIN or errno.EINPROGRESS.  Instead, returns 0
        and a new Stream.  The connect() method can be used to check for
        successful connection completion."""
        cls = Stream._find_method(name)
        if not cls:
            return errno.EAFNOSUPPORT, None

        suffix = name.split(":", 1)[1]
        if name.startswith("unix:"):
            suffix = ovs.util.abs_file_name(ovs.dirs.RUNDIR, suffix)
        error, sock = cls._open(suffix, dscp)
        if error:
            return error, None
        else:
            status = ovs.socket_util.check_connection_completion(sock)
            return 0, Stream(sock, name, status)

    @staticmethod
    def _open(suffix, dscp):
        raise NotImplementedError("This method must be overrided by subclass")

    @staticmethod
    def open_block((error, stream)):
        """Blocks until a Stream completes its connection attempt, either
        succeeding or failing.  (error, stream) should be the tuple returned by
        Stream.open().  Returns a tuple of the same form.

        Typical usage:
        error, stream = Stream.open_block(Stream.open("unix:/tmp/socket"))"""

        if not error:
            while True:
                error = stream.connect()
                if error != errno.EAGAIN:
                    break
                stream.run()
                poller = ovs.poller.Poller()
                stream.run_wait(poller)
                stream.connect_wait(poller)
                poller.block()
            assert error != errno.EINPROGRESS

        if error and stream:
            stream.close()
            stream = None
        return error, stream

    def close(self):
        self.socket.close()

    def __scs_connecting(self):
        retval = ovs.socket_util.check_connection_completion(self.socket)
        assert retval != errno.EINPROGRESS
        if retval == 0:
            self.state = Stream.__S_CONNECTED
        elif retval != errno.EAGAIN:
            self.state = Stream.__S_DISCONNECTED
            self.error = retval

    def connect(self):
        """Tries to complete the connection on this stream.  If the connection
        is complete, returns 0 if the connection was successful or a positive
        errno value if it failed.  If the connection is still in progress,
        returns errno.EAGAIN."""

        if self.state == Stream.__S_CONNECTING:
            self.__scs_connecting()

        if self.state == Stream.__S_CONNECTING:
            return errno.EAGAIN
        elif self.state == Stream.__S_CONNECTED:
            return 0
        else:
            assert self.state == Stream.__S_DISCONNECTED
            return self.error

    def recv(self, n):
        """Tries to receive up to 'n' bytes from this stream.  Returns a
        (error, string) tuple:

            - If successful, 'error' is zero and 'string' contains between 1
              and 'n' bytes of data.

            - On error, 'error' is a positive errno value.

            - If the connection has been closed in the normal fashion or if 'n'
              is 0, the tuple is (0, "").

        The recv function will not block waiting for data to arrive.  If no
        data have been received, it returns (errno.EAGAIN, "") immediately."""

        retval = self.connect()
        if retval != 0:
            return (retval, "")
        elif n == 0:
            return (0, "")

        try:
            return (0, self.socket.recv(n))
        except socket.error, e:
            return (ovs.socket_util.get_exception_errno(e), "")

    def send(self, buf):
        """Tries to send 'buf' on this stream.

        If successful, returns the number of bytes sent, between 1 and
        len(buf).  0 is only a valid return value if len(buf) is 0.

        On error, returns a negative errno value.

        Will not block.  If no bytes can be immediately accepted for
        transmission, returns -errno.EAGAIN immediately."""

        retval = self.connect()
        if retval != 0:
            return -retval
        elif len(buf) == 0:
            return 0

        try:
            return self.socket.send(buf)
        except socket.error, e:
            return -ovs.socket_util.get_exception_errno(e)

    def run(self):
        pass

    def run_wait(self, poller):
        pass

    def wait(self, poller, wait):
        assert wait in (Stream.W_CONNECT, Stream.W_RECV, Stream.W_SEND)

        if self.state == Stream.__S_DISCONNECTED:
            poller.immediate_wake()
            return

        if self.state == Stream.__S_CONNECTING:
            wait = Stream.W_CONNECT
        if wait == Stream.W_RECV:
            poller.fd_wait(self.socket, ovs.poller.POLLIN)
        else:
            poller.fd_wait(self.socket, ovs.poller.POLLOUT)

    def connect_wait(self, poller):
        self.wait(poller, Stream.W_CONNECT)

    def recv_wait(self, poller):
        self.wait(poller, Stream.W_RECV)

    def send_wait(self, poller):
        self.wait(poller, Stream.W_SEND)

    def __del__(self):
        # Don't delete the file: we might have forked.
        self.socket.close()


class PassiveStream(object):
    @staticmethod
    def is_valid_name(name):
        """Returns True if 'name' is a passive stream name in the form
        "TYPE:ARGS" and TYPE is a supported passive stream type (currently only
        "punix:"), otherwise False."""
        return name.startswith("punix:")

    def __init__(self, sock, name, bind_path):
        self.name = name
        self.socket = sock
        self.bind_path = bind_path

    @staticmethod
    def open(name):
        """Attempts to start listening for remote stream connections.  'name'
        is a connection name in the form "TYPE:ARGS", where TYPE is an passive
        stream class's name and ARGS are stream class-specific.  Currently the
        only supported TYPE is "punix".

        Returns (error, pstream): on success 'error' is 0 and 'pstream' is the
        new PassiveStream, on failure 'error' is a positive errno value and
        'pstream' is None."""
        if not PassiveStream.is_valid_name(name):
            return errno.EAFNOSUPPORT, None

        bind_path = name[6:]
        if name.startswith("punix:"):
            bind_path = ovs.util.abs_file_name(ovs.dirs.RUNDIR, bind_path)
        error, sock = ovs.socket_util.make_unix_socket(socket.SOCK_STREAM,
                                                       True, bind_path, None)
        if error:
            return error, None

        try:
            sock.listen(10)
        except socket.error, e:
            vlog.err("%s: listen: %s" % (name, os.strerror(e.error)))
            sock.close()
            return e.error, None

        return 0, PassiveStream(sock, name, bind_path)

    def close(self):
        """Closes this PassiveStream."""
        self.socket.close()
        if self.bind_path is not None:
            ovs.fatal_signal.unlink_file_now(self.bind_path)
            self.bind_path = None

    def accept(self):
        """Tries to accept a new connection on this passive stream.  Returns
        (error, stream): if successful, 'error' is 0 and 'stream' is the new
        Stream object, and on failure 'error' is a positive errno value and
        'stream' is None.

        Will not block waiting for a connection.  If no connection is ready to
        be accepted, returns (errno.EAGAIN, None) immediately."""

        while True:
            try:
                sock, addr = self.socket.accept()
                ovs.socket_util.set_nonblocking(sock)
                return 0, Stream(sock, "unix:%s" % addr, 0)
            except socket.error, e:
                error = ovs.socket_util.get_exception_errno(e)
                if error != errno.EAGAIN:
                    # XXX rate-limit
                    vlog.dbg("accept: %s" % os.strerror(error))
                return error, None

    def wait(self, poller):
        poller.fd_wait(self.socket, ovs.poller.POLLIN)

    def __del__(self):
        # Don't delete the file: we might have forked.
        self.socket.close()


def usage(name):
    return """
Active %s connection methods:
  unix:FILE               Unix domain socket named FILE
  tcp:IP:PORT             TCP socket to IP with port no of PORT

Passive %s connection methods:
  punix:FILE              Listen on Unix domain socket FILE""" % (name, name)


class UnixStream(Stream):
    @staticmethod
    def _open(suffix, dscp):
        connect_path = suffix
        return  ovs.socket_util.make_unix_socket(socket.SOCK_STREAM,
                                                 True, None, connect_path)
Stream.register_method("unix", UnixStream)


class TCPStream(Stream):
    @staticmethod
    def _open(suffix, dscp):
        error, sock = ovs.socket_util.inet_open_active(socket.SOCK_STREAM,
                                                       suffix, 0, dscp)
        if not error:
            sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        return error, sock
Stream.register_method("tcp", TCPStream)



# Copyright (c) 2009, 2010, 2011, 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import getopt
import re
import os
import signal
import sys
import uuid

from ovs.db import error
import ovs.db.idl
import ovs.db.schema
from ovs.db import data
from ovs.db import types
import ovs.ovsuuid
import ovs.poller
import ovs.util


def unbox_json(json):
    if type(json) == list and len(json) == 1:
        return json[0]
    else:
        return json


def do_default_atoms():
    for type_ in types.ATOMIC_TYPES:
        if type_ == types.VoidType:
            continue

        sys.stdout.write("%s: " % type_.to_string())

        atom = data.Atom.default(type_)
        if atom != data.Atom.default(type_):
            sys.stdout.write("wrong\n")
            sys.exit(1)

        sys.stdout.write("OK\n")


def do_default_data():
    any_errors = False
    for n_min in 0, 1:
        for key in types.ATOMIC_TYPES:
            if key == types.VoidType:
                continue
            for value in types.ATOMIC_TYPES:
                if value == types.VoidType:
                    valueBase = None
                else:
                    valueBase = types.BaseType(value)
                type_ = types.Type(types.BaseType(key), valueBase, n_min, 1)
                assert type_.is_valid()

                sys.stdout.write("key %s, value %s, n_min %d: "
                                 % (key.to_string(), value.to_string(), n_min))

                datum = data.Datum.default(type_)
                if datum != data.Datum.default(type_):
                    sys.stdout.write("wrong\n")
                    any_errors = True
                else:
                    sys.stdout.write("OK\n")
    if any_errors:
        sys.exit(1)


def do_parse_atomic_type(type_string):
    type_json = unbox_json(ovs.json.from_string(type_string))
    atomic_type = types.AtomicType.from_json(type_json)
    print ovs.json.to_string(atomic_type.to_json(), sort_keys=True)


def do_parse_base_type(type_string):
    type_json = unbox_json(ovs.json.from_string(type_string))
    base_type = types.BaseType.from_json(type_json)
    print ovs.json.to_string(base_type.to_json(), sort_keys=True)


def do_parse_type(type_string):
    type_json = unbox_json(ovs.json.from_string(type_string))
    type_ = types.Type.from_json(type_json)
    print ovs.json.to_string(type_.to_json(), sort_keys=True)


def do_parse_atoms(type_string, *atom_strings):
    type_json = unbox_json(ovs.json.from_string(type_string))
    base = types.BaseType.from_json(type_json)
    for atom_string in atom_strings:
        atom_json = unbox_json(ovs.json.from_string(atom_string))
        try:
            atom = data.Atom.from_json(base, atom_json)
            print ovs.json.to_string(atom.to_json())
        except error.Error, e:
            print unicode(e)


def do_parse_data(type_string, *data_strings):
    type_json = unbox_json(ovs.json.from_string(type_string))
    type_ = types.Type.from_json(type_json)
    for datum_string in data_strings:
        datum_json = unbox_json(ovs.json.from_string(datum_string))
        datum = data.Datum.from_json(type_, datum_json)
        print ovs.json.to_string(datum.to_json())


def do_sort_atoms(type_string, atom_strings):
    type_json = unbox_json(ovs.json.from_string(type_string))
    base = types.BaseType.from_json(type_json)
    atoms = [data.Atom.from_json(base, atom_json)
             for atom_json in unbox_json(ovs.json.from_string(atom_strings))]
    print ovs.json.to_string([data.Atom.to_json(atom)
                              for atom in sorted(atoms)])


def do_parse_column(name, column_string):
    column_json = unbox_json(ovs.json.from_string(column_string))
    column = ovs.db.schema.ColumnSchema.from_json(column_json, name)
    print ovs.json.to_string(column.to_json(), sort_keys=True)


def do_parse_table(name, table_string, default_is_root_string='false'):
    default_is_root = default_is_root_string == 'true'
    table_json = unbox_json(ovs.json.from_string(table_string))
    table = ovs.db.schema.TableSchema.from_json(table_json, name)
    print ovs.json.to_string(table.to_json(default_is_root), sort_keys=True)


def do_parse_schema(schema_string):
    schema_json = unbox_json(ovs.json.from_string(schema_string))
    schema = ovs.db.schema.DbSchema.from_json(schema_json)
    print ovs.json.to_string(schema.to_json(), sort_keys=True)


def print_idl(idl, step):
    simple = idl.tables["simple"].rows
    l1 = idl.tables["link1"].rows
    l2 = idl.tables["link2"].rows

    n = 0
    for row in simple.itervalues():
        s = ("%03d: i=%s r=%s b=%s s=%s u=%s "
             "ia=%s ra=%s ba=%s sa=%s ua=%s uuid=%s"
             % (step, row.i, row.r, row.b, row.s, row.u,
                row.ia, row.ra, row.ba, row.sa, row.ua, row.uuid))
        s = re.sub('""|,|u?\'', "", s)
        s = re.sub('UUID\(([^)]+)\)', r'\1', s)
        s = re.sub('False', 'false', s)
        s = re.sub('True', 'true', s)
        s = re.sub(r'(ba)=([^[][^ ]*) ', r'\1=[\2] ', s)
        print(s)
        n += 1

    for row in l1.itervalues():
        s = ["%03d: i=%s k=" % (step, row.i)]
        if row.k:
            s.append(str(row.k.i))
        s.append(" ka=[")
        s.append(' '.join(sorted(str(ka.i) for ka in row.ka)))
        s.append("] l2=")
        if row.l2:
            s.append(str(row.l2[0].i))
        s.append(" uuid=%s" % row.uuid)
        print(''.join(s))
        n += 1

    for row in l2.itervalues():
        s = ["%03d: i=%s l1=" % (step, row.i)]
        if row.l1:
            s.append(str(row.l1[0].i))
        s.append(" uuid=%s" % row.uuid)
        print(''.join(s))
        n += 1

    if not n:
        print("%03d: empty" % step)
    sys.stdout.flush()


def substitute_uuids(json, symtab):
    if type(json) in [str, unicode]:
        symbol = symtab.get(json)
        if symbol:
            return str(symbol)
    elif type(json) == list:
        return [substitute_uuids(element, symtab) for element in json]
    elif type(json) == dict:
        d = {}
        for key, value in json.iteritems():
            d[key] = substitute_uuids(value, symtab)
        return d
    return json


def parse_uuids(json, symtab):
    if type(json) in [str, unicode] and ovs.ovsuuid.is_valid_string(json):
        name = "#%d#" % len(symtab)
        sys.stderr.write("%s = %s\n" % (name, json))
        symtab[name] = json
    elif type(json) == list:
        for element in json:
            parse_uuids(element, symtab)
    elif type(json) == dict:
        for value in json.itervalues():
            parse_uuids(value, symtab)


def idltest_find_simple(idl, i):
    for row in idl.tables["simple"].rows.itervalues():
        if row.i == i:
            return row
    return None


def idl_set(idl, commands, step):
    txn = ovs.db.idl.Transaction(idl)
    increment = False
    for command in commands.split(','):
        words = command.split()
        name = words[0]
        args = words[1:]

        if name == "set":
            if len(args) != 3:
                sys.stderr.write('"set" command requires 3 arguments\n')
                sys.exit(1)

            s = idltest_find_simple(idl, int(args[0]))
            if not s:
                sys.stderr.write('"set" command asks for nonexistent i=%d\n'
                                 % int(args[0]))
                sys.exit(1)

            if args[1] == "b":
                s.b = args[2] == "1"
            elif args[1] == "s":
                s.s = args[2]
            elif args[1] == "u":
                s.u = uuid.UUID(args[2])
            elif args[1] == "r":
                s.r = float(args[2])
            else:
                sys.stderr.write('"set" comamnd asks for unknown column %s\n'
                                 % args[2])
                sys.stderr.exit(1)
        elif name == "insert":
            if len(args) != 1:
                sys.stderr.write('"set" command requires 1 argument\n')
                sys.exit(1)

            s = txn.insert(idl.tables["simple"])
            s.i = int(args[0])
        elif name == "delete":
            if len(args) != 1:
                sys.stderr.write('"delete" command requires 1 argument\n')
                sys.exit(1)

            s = idltest_find_simple(idl, int(args[0]))
            if not s:
                sys.stderr.write('"delete" command asks for nonexistent i=%d\n'
                                 % int(args[0]))
                sys.exit(1)
            s.delete()
        elif name == "verify":
            if len(args) != 2:
                sys.stderr.write('"verify" command requires 2 arguments\n')
                sys.exit(1)

            s = idltest_find_simple(idl, int(args[0]))
            if not s:
                sys.stderr.write('"verify" command asks for nonexistent i=%d\n'
                                 % int(args[0]))
                sys.exit(1)

            if args[1] in ("i", "b", "s", "u", "r"):
                s.verify(args[1])
            else:
                sys.stderr.write('"verify" command asks for unknown column '
                                 '"%s"\n' % args[1])
                sys.exit(1)
        elif name == "increment":
            if len(args) != 1:
                sys.stderr.write('"increment" command requires 1 argument\n')
                sys.exit(1)

            s = idltest_find_simple(idl, int(args[0]))
            if not s:
                sys.stderr.write('"set" command asks for nonexistent i=%d\n'
                                 % int(args[0]))
                sys.exit(1)

            s.increment("i")
            increment = True
        elif name == "abort":
            txn.abort()
            break
        elif name == "destroy":
            print "%03d: destroy" % step
            sys.stdout.flush()
            txn.abort()
            return
        elif name == "linktest":
            l1_0 = txn.insert(idl.tables["link1"])
            l1_0.i = 1
            l1_0.k = [l1_0]
            l1_0.ka = [l1_0]
            l1_1 = txn.insert(idl.tables["link1"])
            l1_1.i = 2
            l1_1.k = [l1_0]
            l1_1.ka = [l1_0, l1_1]
        elif name == 'getattrtest':
            l1 = txn.insert(idl.tables["link1"])
            i = getattr(l1, 'i', 1)
            assert i == 1
            l1.i = 2
            i = getattr(l1, 'i', 1)
            assert i == 2
            l1.k = [l1]
        else:
            sys.stderr.write("unknown command %s\n" % name)
            sys.exit(1)

    status = txn.commit_block()
    sys.stdout.write("%03d: commit, status=%s"
                     % (step, ovs.db.idl.Transaction.status_to_string(status)))
    if increment and status == ovs.db.idl.Transaction.SUCCESS:
        sys.stdout.write(", increment=%d" % txn.get_increment_new_value())
    sys.stdout.write("\n")
    sys.stdout.flush()


def do_idl(schema_file, remote, *commands):
    schema_helper = ovs.db.idl.SchemaHelper(schema_file)
    schema_helper.register_all()
    idl = ovs.db.idl.Idl(remote, schema_helper)

    if commands:
        error, stream = ovs.stream.Stream.open_block(
            ovs.stream.Stream.open(remote))
        if error:
            sys.stderr.write("failed to connect to \"%s\"" % remote)
            sys.exit(1)
        rpc = ovs.jsonrpc.Connection(stream)
    else:
        rpc = None

    symtab = {}
    seqno = 0
    step = 0
    for command in commands:
        if command.startswith("+"):
            # The previous transaction didn't change anything.
            command = command[1:]
        else:
            # Wait for update.
            while idl.change_seqno == seqno and not idl.run():
                rpc.run()

                poller = ovs.poller.Poller()
                idl.wait(poller)
                rpc.wait(poller)
                poller.block()

            print_idl(idl, step)
            step += 1

        seqno = idl.change_seqno

        if command == "reconnect":
            print("%03d: reconnect" % step)
            sys.stdout.flush()
            step += 1
            idl.force_reconnect()
        elif not command.startswith("["):
            idl_set(idl, command, step)
            step += 1
        else:
            json = ovs.json.from_string(command)
            if type(json) in [str, unicode]:
                sys.stderr.write("\"%s\": %s\n" % (command, json))
                sys.exit(1)
            json = substitute_uuids(json, symtab)
            request = ovs.jsonrpc.Message.create_request("transact", json)
            error, reply = rpc.transact_block(request)
            if error:
                sys.stderr.write("jsonrpc transaction failed: %s"
                                 % os.strerror(error))
                sys.exit(1)
            elif reply.error is not None:
                sys.stderr.write("jsonrpc transaction failed: %s"
                                 % reply.error)
                sys.exit(1)

            sys.stdout.write("%03d: " % step)
            sys.stdout.flush()
            step += 1
            if reply.result is not None:
                parse_uuids(reply.result, symtab)
            reply.id = None
            sys.stdout.write("%s\n" % ovs.json.to_string(reply.to_json()))
            sys.stdout.flush()

    if rpc:
        rpc.close()
    while idl.change_seqno == seqno and not idl.run():
        poller = ovs.poller.Poller()
        idl.wait(poller)
        poller.block()
    print_idl(idl, step)
    step += 1
    idl.close()
    print("%03d: done" % step)


def usage():
    print """\
%(program_name)s: test utility for Open vSwitch database Python bindings
usage: %(program_name)s [OPTIONS] COMMAND ARG...

The following commands are supported:
default-atoms
  test ovsdb_atom_default()
default-data
  test ovsdb_datum_default()
parse-atomic-type TYPE
  parse TYPE as OVSDB atomic type, and re-serialize
parse-base-type TYPE
  parse TYPE as OVSDB base type, and re-serialize
parse-type JSON
  parse JSON as OVSDB type, and re-serialize
parse-atoms TYPE ATOM...
  parse JSON ATOMs as atoms of TYPE, and re-serialize
parse-atom-strings TYPE ATOM...
  parse string ATOMs as atoms of given TYPE, and re-serialize
sort-atoms TYPE ATOM...
  print JSON ATOMs in sorted order
parse-data TYPE DATUM...
  parse JSON DATUMs as data of given TYPE, and re-serialize
parse-column NAME OBJECT
  parse column NAME with info OBJECT, and re-serialize
parse-table NAME OBJECT [DEFAULT-IS-ROOT]
  parse table NAME with info OBJECT
parse-schema JSON
  parse JSON as an OVSDB schema, and re-serialize
idl SCHEMA SERVER [TRANSACTION...]
  connect to SERVER (which has the specified SCHEMA) and dump the
  contents of the database as seen initially by the IDL implementation
  and after executing each TRANSACTION.  (Each TRANSACTION must modify
  the database or this command will hang.)

The following options are also available:
  -t, --timeout=SECS          give up after SECS seconds
  -h, --help                  display this help message\
""" % {'program_name': ovs.util.PROGRAM_NAME}
    sys.exit(0)


def main(argv):
    try:
        options, args = getopt.gnu_getopt(argv[1:], 't:h',
                                          ['timeout',
                                           'help'])
    except getopt.GetoptError, geo:
        sys.stderr.write("%s: %s\n" % (ovs.util.PROGRAM_NAME, geo.msg))
        sys.exit(1)

    for key, value in options:
        if key in ['-h', '--help']:
            usage()
        elif key in ['-t', '--timeout']:
            try:
                timeout = int(value)
                if timeout < 1:
                    raise TypeError
            except TypeError:
                raise error.Error("value %s on -t or --timeout is not at "
                                  "least 1" % value)
            signal.alarm(timeout)
        else:
            sys.exit(0)

    if not args:
        sys.stderr.write("%s: missing command argument "
                         "(use --help for help)\n" % ovs.util.PROGRAM_NAME)
        sys.exit(1)

    commands = {"default-atoms": (do_default_atoms, 0),
                "default-data": (do_default_data, 0),
                "parse-atomic-type": (do_parse_atomic_type, 1),
                "parse-base-type": (do_parse_base_type, 1),
                "parse-type": (do_parse_type, 1),
                "parse-atoms": (do_parse_atoms, (2,)),
                "parse-data": (do_parse_data, (2,)),
                "sort-atoms": (do_sort_atoms, 2),
                "parse-column": (do_parse_column, 2),
                "parse-table": (do_parse_table, (2, 3)),
                "parse-schema": (do_parse_schema, 1),
                "idl": (do_idl, (2,))}

    command_name = args[0]
    args = args[1:]
    if not command_name in commands:
        sys.stderr.write("%s: unknown command \"%s\" "
                         "(use --help for help)\n" % (ovs.util.PROGRAM_NAME,
                                                      command_name))
        sys.exit(1)

    func, n_args = commands[command_name]
    if type(n_args) == tuple:
        if len(args) < n_args[0]:
            sys.stderr.write("%s: \"%s\" requires at least %d arguments but "
                             "only %d provided\n"
                             % (ovs.util.PROGRAM_NAME, command_name,
                                n_args, len(args)))
            sys.exit(1)
    elif type(n_args) == int:
        if len(args) != n_args:
            sys.stderr.write("%s: \"%s\" requires %d arguments but %d "
                             "provided\n"
                             % (ovs.util.PROGRAM_NAME, command_name,
                                n_args, len(args)))
            sys.exit(1)
    else:
        assert False

    func(*args)


if __name__ == '__main__':
    try:
        main(sys.argv)
    except error.Error, e:
        sys.stderr.write("%s\n" % e)
        sys.exit(1)



import os
PKGDATADIR = os.environ.get("OVS_PKGDATADIR", """/usr/local/share/openvswitch""")
RUNDIR = os.environ.get("OVS_RUNDIR", """/var/run""")
LOGDIR = os.environ.get("OVS_LOGDIR", """/usr/local/var/log""")
BINDIR = os.environ.get("OVS_BINDIR", """/usr/local/bin""")

DBDIR = os.environ.get("OVS_DBDIR")
if not DBDIR:
    sysconfdir = os.environ.get("OVS_SYSCONFDIR")
    if sysconfdir:
        DBDIR = "%s/openvswitch" % sysconfdir
    else:
        DBDIR = """/usr/local/etc/openvswitch"""




# Copyright (c) 2011, 2012, 2013 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import logging
import logging.handlers
import re
import socket
import sys

import ovs.dirs
import ovs.unixctl
import ovs.util

FACILITIES = {"console": "info", "file": "info", "syslog": "info"}
LEVELS = {
    "dbg": logging.DEBUG,
    "info": logging.INFO,
    "warn": logging.WARNING,
    "err": logging.ERROR,
    "emer": logging.CRITICAL,
    "off": logging.CRITICAL
}


def get_level(level_str):
    return LEVELS.get(level_str.lower())


class Vlog:
    __inited = False
    __msg_num = 0
    __mfl = {}  # Module -> facility -> level
    __log_file = None
    __file_handler = None

    def __init__(self, name):
        """Creates a new Vlog object representing a module called 'name'.  The
        created Vlog object will do nothing until the Vlog.init() static method
        is called.  Once called, no more Vlog objects may be created."""

        assert not Vlog.__inited
        self.name = name.lower()
        if name not in Vlog.__mfl:
            Vlog.__mfl[self.name] = FACILITIES.copy()

    def __log(self, level, message, **kwargs):
        if not Vlog.__inited:
            return

        dt = datetime.datetime.utcnow();
        now = dt.strftime("%Y-%m-%dT%H:%M:%S.%%03iZ") % (dt.microsecond/1000)
        syslog_message = ("%s|%s|%s|%s"
                           % (Vlog.__msg_num, self.name, level, message))

        level = LEVELS.get(level.lower(), logging.DEBUG)
        Vlog.__msg_num += 1

        for f, f_level in Vlog.__mfl[self.name].iteritems():
            f_level = LEVELS.get(f_level, logging.CRITICAL)
            if level >= f_level:
                if f == "syslog":
                    message = "ovs|" + syslog_message
                else:
                    message = "%s|%s" % (now, syslog_message)
                logging.getLogger(f).log(level, message, **kwargs)

    def emer(self, message, **kwargs):
        self.__log("EMER", message, **kwargs)

    def err(self, message, **kwargs):
        self.__log("ERR", message, **kwargs)

    def warn(self, message, **kwargs):
        self.__log("WARN", message, **kwargs)

    def info(self, message, **kwargs):
        self.__log("INFO", message, **kwargs)

    def dbg(self, message, **kwargs):
        self.__log("DBG", message, **kwargs)

    def exception(self, message):
        """Logs 'message' at ERR log level.  Includes a backtrace when in
        exception context."""
        self.err(message, exc_info=True)

    @staticmethod
    def init(log_file=None):
        """Intializes the Vlog module.  Causes Vlog to write to 'log_file' if
        not None.  Should be called after all Vlog objects have been created.
        No logging will occur until this function is called."""

        if Vlog.__inited:
            return

        Vlog.__inited = True
        logging.raiseExceptions = False
        Vlog.__log_file = log_file
        for f in FACILITIES:
            logger = logging.getLogger(f)
            logger.setLevel(logging.DEBUG)

            try:
                if f == "console":
                    logger.addHandler(logging.StreamHandler(sys.stderr))
                elif f == "syslog":
                    logger.addHandler(logging.handlers.SysLogHandler(
                        address="/dev/log",
                        facility=logging.handlers.SysLogHandler.LOG_DAEMON))
                elif f == "file" and Vlog.__log_file:
                    Vlog.__file_handler = logging.FileHandler(Vlog.__log_file)
                    logger.addHandler(Vlog.__file_handler)
            except (IOError, socket.error):
                logger.setLevel(logging.CRITICAL)

        ovs.unixctl.command_register("vlog/reopen", "", 0, 0,
                                     Vlog._unixctl_vlog_reopen, None)
        ovs.unixctl.command_register("vlog/set", "spec", 1, sys.maxint,
                                     Vlog._unixctl_vlog_set, None)
        ovs.unixctl.command_register("vlog/list", "", 0, 0,
                                     Vlog._unixctl_vlog_list, None)

    @staticmethod
    def set_level(module, facility, level):
        """ Sets the log level of the 'module'-'facility' tuple to 'level'.
        All three arguments are strings which are interpreted the same as
        arguments to the --verbose flag.  Should be called after all Vlog
        objects have already been created."""

        module = module.lower()
        facility = facility.lower()
        level = level.lower()

        if facility != "any" and facility not in FACILITIES:
            return

        if module != "any" and module not in Vlog.__mfl:
            return

        if level not in LEVELS:
            return

        if module == "any":
            modules = Vlog.__mfl.keys()
        else:
            modules = [module]

        if facility == "any":
            facilities = FACILITIES.keys()
        else:
            facilities = [facility]

        for m in modules:
            for f in facilities:
                Vlog.__mfl[m][f] = level

    @staticmethod
    def set_levels_from_string(s):
        module = None
        level = None
        facility = None

        for word in [w.lower() for w in re.split('[ :]', s)]:
            if word == "any":
                pass
            elif word in FACILITIES:
                if facility:
                    return "cannot specify multiple facilities"
                facility = word
            elif word in LEVELS:
                if level:
                    return "cannot specify multiple levels"
                level = word
            elif word in Vlog.__mfl:
                if module:
                    return "cannot specify multiple modules"
                module = word
            else:
                return "no facility, level, or module \"%s\"" % word

        Vlog.set_level(module or "any", facility or "any", level or "any")

    @staticmethod
    def get_levels():
        lines = ["                 console    syslog    file\n",
                 "                 -------    ------    ------\n"]
        lines.extend(sorted(["%-16s  %4s       %4s       %4s\n"
                             % (m,
                                Vlog.__mfl[m]["console"],
                                Vlog.__mfl[m]["syslog"],
                                Vlog.__mfl[m]["file"]) for m in Vlog.__mfl]))
        return ''.join(lines)

    @staticmethod
    def reopen_log_file():
        """Closes and then attempts to re-open the current log file.  (This is
        useful just after log rotation, to ensure that the new log file starts
        being used.)"""

        if Vlog.__log_file:
            logger = logging.getLogger("file")
            logger.removeHandler(Vlog.__file_handler)
            Vlog.__file_handler = logging.FileHandler(Vlog.__log_file)
            logger.addHandler(Vlog.__file_handler)

    @staticmethod
    def _unixctl_vlog_reopen(conn, unused_argv, unused_aux):
        if Vlog.__log_file:
            Vlog.reopen_log_file()
            conn.reply(None)
        else:
            conn.reply("Logging to file not configured")

    @staticmethod
    def _unixctl_vlog_set(conn, argv, unused_aux):
        for arg in argv:
            msg = Vlog.set_levels_from_string(arg)
            if msg:
                conn.reply(msg)
                return
        conn.reply(None)

    @staticmethod
    def _unixctl_vlog_list(conn, unused_argv, unused_aux):
        conn.reply(Vlog.get_levels())

def add_args(parser):
    """Adds vlog related options to 'parser', an ArgumentParser object.  The
    resulting arguments parsed by 'parser' should be passed to handle_args."""

    group = parser.add_argument_group(title="Logging Options")
    group.add_argument("--log-file", nargs="?", const="default",
                       help="Enables logging to a file.  Default log file"
                       " is used if LOG_FILE is omitted.")
    group.add_argument("-v", "--verbose", nargs="*",
                       help="Sets logging levels, see ovs-vswitchd(8)."
                       "  Defaults to dbg.")


def handle_args(args):
    """ Handles command line arguments ('args') parsed by an ArgumentParser.
    The ArgumentParser should have been primed by add_args().  Also takes care
    of initializing the Vlog module."""

    log_file = args.log_file
    if log_file == "default":
        log_file = "%s/%s.log" % (ovs.dirs.LOGDIR, ovs.util.PROGRAM_NAME)

    if args.verbose is None:
        args.verbose = []
    elif args.verbose == []:
        args.verbose = ["any:any:dbg"]

    for verbose in args.verbose:
        msg = Vlog.set_levels_from_string(verbose)
        if msg:
            ovs.util.ovs_fatal(0, "processing \"%s\": %s" % (verbose, msg))

    Vlog.init(log_file)



# Copyright (c) 2010, 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import errno
import os
import select
import socket
import sys

import ovs.fatal_signal
import ovs.poller
import ovs.vlog

vlog = ovs.vlog.Vlog("socket_util")


def make_unix_socket(style, nonblock, bind_path, connect_path):
    """Creates a Unix domain socket in the given 'style' (either
    socket.SOCK_DGRAM or socket.SOCK_STREAM) that is bound to 'bind_path' (if
    'bind_path' is not None) and connected to 'connect_path' (if 'connect_path'
    is not None).  If 'nonblock' is true, the socket is made non-blocking.

    Returns (error, socket): on success 'error' is 0 and 'socket' is a new
    socket object, on failure 'error' is a positive errno value and 'socket' is
    None."""

    try:
        sock = socket.socket(socket.AF_UNIX, style)
    except socket.error, e:
        return get_exception_errno(e), None

    try:
        if nonblock:
            set_nonblocking(sock)
        if bind_path is not None:
            # Delete bind_path but ignore ENOENT.
            try:
                os.unlink(bind_path)
            except OSError, e:
                if e.errno != errno.ENOENT:
                    return e.errno, None

            ovs.fatal_signal.add_file_to_unlink(bind_path)
            sock.bind(bind_path)

            try:
                if sys.hexversion >= 0x02060000:
                    os.fchmod(sock.fileno(), 0700)
                else:
                    os.chmod("/dev/fd/%d" % sock.fileno(), 0700)
            except OSError, e:
                pass
        if connect_path is not None:
            try:
                sock.connect(connect_path)
            except socket.error, e:
                if get_exception_errno(e) != errno.EINPROGRESS:
                    raise
        return 0, sock
    except socket.error, e:
        sock.close()
        if (bind_path is not None and
            os.path.exists(bind_path)):
            ovs.fatal_signal.unlink_file_now(bind_path)
        eno = ovs.socket_util.get_exception_errno(e)
        if (eno == "AF_UNIX path too long" and
            os.uname()[0] == "Linux"):
            short_connect_path = None
            short_bind_path = None
            connect_dirfd = None
            bind_dirfd = None
            # Try workaround using /proc/self/fd
            if connect_path is not None:
                dirname = os.path.dirname(connect_path)
                basename = os.path.basename(connect_path)
                try:
                    connect_dirfd = os.open(dirname, os.O_DIRECTORY | os.O_RDONLY)
                except OSError, err:
                    return get_exception_errno(err), None
                short_connect_path = "/proc/self/fd/%d/%s" % (connect_dirfd, basename)

            if bind_path is not None:
                dirname = os.path.dirname(bind_path)
                basename = os.path.basename(bind_path)
                try:
                    bind_dirfd = os.open(dirname, os.O_DIRECTORY | os.O_RDONLY)
                except OSError, err:
                    return get_exception_errno(err), None
                short_bind_path = "/proc/self/fd/%d/%s" % (bind_dirfd, basename)

            try:
                return make_unix_socket(style, nonblock, short_bind_path, short_connect_path)
            finally:
                if connect_dirfd is not None:
                    os.close(connect_dirfd)
                if bind_dirfd is not None:
                    os.close(bind_dirfd)
        else:
            return get_exception_errno(e), None


def check_connection_completion(sock):
    p = ovs.poller.SelectPoll()
    p.register(sock, ovs.poller.POLLOUT)
    pfds = p.poll(0)
    if len(pfds) == 1:
        revents = pfds[0][1]
        if revents & ovs.poller.POLLERR:
            try:
                # The following should raise an exception.
                socket.send("\0", socket.MSG_DONTWAIT)

                # (Here's where we end up if it didn't.)
                # XXX rate-limit
                vlog.err("poll return POLLERR but send succeeded")
                return errno.EPROTO
            except socket.error, e:
                return get_exception_errno(e)
        else:
            return 0
    else:
        return errno.EAGAIN


def inet_parse_active(target, default_port):
    address = target.split(":")
    host_name = address[0]
    if not host_name:
        raise ValueError("%s: bad peer name format" % target)
    if len(address) >= 2:
        port = int(address[1])
    elif default_port:
        port = default_port
    else:
        raise ValueError("%s: port number must be specified" % target)
    return (host_name, port)


def inet_open_active(style, target, default_port, dscp):
    address = inet_parse_active(target, default_port)
    try:
        sock = socket.socket(socket.AF_INET, style, 0)
    except socket.error, e:
        return get_exception_errno(e), None

    try:
        set_nonblocking(sock)
        set_dscp(sock, dscp)
        try:
            sock.connect(address)
        except socket.error, e:
            if get_exception_errno(e) != errno.EINPROGRESS:
                raise
        return 0, sock
    except socket.error, e:
        sock.close()
        return get_exception_errno(e), None


def get_exception_errno(e):
    """A lot of methods on Python socket objects raise socket.error, but that
    exception is documented as having two completely different forms of
    arguments: either a string or a (errno, string) tuple.  We only want the
    errno."""
    if type(e.args) == tuple:
        return e.args[0]
    else:
        return errno.EPROTO


null_fd = -1


def get_null_fd():
    """Returns a readable and writable fd for /dev/null, if successful,
    otherwise a negative errno value.  The caller must not close the returned
    fd (because the same fd will be handed out to subsequent callers)."""
    global null_fd
    if null_fd < 0:
        try:
            null_fd = os.open("/dev/null", os.O_RDWR)
        except OSError, e:
            vlog.err("could not open /dev/null: %s" % os.strerror(e.errno))
            return -e.errno
    return null_fd


def write_fully(fd, buf):
    """Returns an (error, bytes_written) tuple where 'error' is 0 on success,
    otherwise a positive errno value, and 'bytes_written' is the number of
    bytes that were written before the error occurred.  'error' is 0 if and
    only if 'bytes_written' is len(buf)."""
    bytes_written = 0
    if len(buf) == 0:
        return 0, 0
    while True:
        try:
            retval = os.write(fd, buf)
            assert retval >= 0
            if retval == len(buf):
                return 0, bytes_written + len(buf)
            elif retval == 0:
                vlog.warn("write returned 0")
                return errno.EPROTO, bytes_written
            else:
                bytes_written += retval
                buf = buf[:retval]
        except OSError, e:
            return e.errno, bytes_written


def set_nonblocking(sock):
    try:
        sock.setblocking(0)
    except socket.error, e:
        vlog.err("could not set nonblocking mode on socket: %s"
                 % os.strerror(get_exception_errno(e)))


def set_dscp(sock, dscp):
    if dscp > 63:
        raise ValueError("Invalid dscp %d" % dscp)
    val = dscp << 2
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_TOS, val)
